@@ -120,7 +120,7 @@ mlt_field mlt_field_new( mlt_multitrack multitrack, mlt_tractor tractor )
 
 mlt_service mlt_field_service( mlt_field self )
 {
-	return MLT_TRACTOR_SERVICE( self->tractor );
+	return self != NULL ? MLT_TRACTOR_SERVICE( self->tractor ) : NULL;
 }
 
 /** Get the multitrack.
@@ -183,7 +183,7 @@ int mlt_field_plant_filter( mlt_field self, mlt_filter that, int track )
 		mlt_tractor_connect( self->tractor, self->producer );
 
 		// Fire an event
-		mlt_events_fire( mlt_field_properties( self ), "service-changed", NULL );
+		mlt_events_fire( mlt_field_properties( self ), "service-changed", mlt_event_data_none() );
 	}
 
 	return result;
@@ -214,7 +214,7 @@ int mlt_field_plant_transition( mlt_field self, mlt_transition that, int a_track
 		mlt_tractor_connect( self->tractor, self->producer );
 
 		// Fire an event
-		mlt_events_fire( mlt_field_properties( self ), "service-changed", NULL );
+		mlt_events_fire( mlt_field_properties( self ), "service-changed", mlt_event_data_none() );
 	}
 
 	return result;
@@ -250,20 +250,20 @@ void mlt_field_disconnect_service( mlt_field self, mlt_service service )
 	int i;
 	switch ( mlt_service_identify(c) )
 	{
-		case filter_type:
+		case mlt_service_filter_type:
 			i = mlt_filter_get_track( MLT_FILTER(c) );
 			mlt_service_connect_producer( c, p, i );
 			break;
-		case transition_type:
+		case mlt_service_transition_type:
 			i = mlt_transition_get_a_track ( MLT_TRANSITION(c) );
 			mlt_service_connect_producer( c, p, i );
 			MLT_TRANSITION(c)->producer = p;
 			break;
-		case tractor_type:
+		case mlt_service_tractor_type:
 			self->producer = p;
 			mlt_tractor_connect( MLT_TRACTOR(c), p );
 		default:
 			break;
 	}
-	mlt_events_fire( mlt_field_properties( self ), "service-changed", NULL );
+	mlt_events_fire( mlt_field_properties( self ), "service-changed", mlt_event_data_none() );
 }
