@@ -3,7 +3,7 @@
  * \brief abstraction for all filter services
  * \see mlt_filter_s
  *
- * Copyright (C) 2003-2019 Meltytech, LLC
+ * Copyright (C) 2003-2020 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -309,7 +309,7 @@ mlt_frame mlt_filter_process( mlt_filter self, mlt_frame frame )
 	// Save the position on the frame
 	mlt_properties_set_position( MLT_FRAME_PROPERTIES( frame ), name, position );
 
-	if ( disable || self->process == NULL )
+	if ( disable || !self || !self->process )
 	{
 		return frame;
 	}
