@@ -3,7 +3,7 @@
  * \brief interface for all frame classes
  * \see mlt_frame_s
  *
- * Copyright (C) 2003-2018 Meltytech, LLC
+ * Copyright (C) 2003-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 #define MLT_FRAME_H
 
 #include "mlt_audio.h"
+#include "mlt_image.h"
 #include "mlt_properties.h"
 #include "mlt_deque.h"
 #include "mlt_service.h"
@@ -58,7 +59,7 @@ typedef int ( *mlt_get_audio )( mlt_frame self, void **buffer, mlt_audio_format
  * \properties \em next \em frame a reference to the unfiltered following frame
  * (no speed factor applied, only available when \em _need_previous_next is set on the producer)
  * \properties \em colorspace the standard for the YUV coefficients
- * \properties \em force_full_luma luma range handling, set to -1 for pass-through, 1 for full range, 0 for scaling
+ * \properties \em force_full_luma luma range handling: 1 for full range, 0 for scaling (DEPRECATED)
  * \properties \em color_trc the color transfer characteristic (gamma)
  * \properties \em audio_frequency the sample rate of the audio
  * \properties \em audio_channels the number of audio channels
@@ -68,18 +69,13 @@ typedef int ( *mlt_get_audio )( mlt_frame self, void **buffer, mlt_audio_format
  * \properties \em width the horizontal resolution of the image
  * \properties \em height the vertical resolution of the image
  * \properties \em aspect_ratio the sample aspect ratio of the image
+ * \properties \em full_range set if the video is full range - only applies to Y'CbCr
  */
 
 struct mlt_frame_s
 {
 	struct mlt_properties_s parent; /**< \private A frame extends properties. */
 
-	/** Get the alpha channel (callback function).
-	 * \param self a frame
-	 * \return the 8-bit alpha channel
-	 */
-	uint8_t * ( *get_alpha_mask )( mlt_frame self );
-
 	/** Convert the image format (callback function).
 	 * \param self a frame
 	 * \param[in,out] image a buffer of image data
@@ -122,8 +118,8 @@ extern int mlt_frame_set_image( mlt_frame self, uint8_t *image, int size, mlt_de
 extern int mlt_frame_set_alpha( mlt_frame self, uint8_t *alpha, int size, mlt_destructor destroy );
 extern void mlt_frame_replace_image( mlt_frame self, uint8_t *image, mlt_image_format format, int width, int height );
 extern int mlt_frame_get_image( mlt_frame self, uint8_t **buffer, mlt_image_format *format, int *width, int *height, int writable );
-extern uint8_t *mlt_frame_get_alpha_mask( mlt_frame self );
 extern uint8_t *mlt_frame_get_alpha( mlt_frame self );
+extern uint8_t *mlt_frame_get_alpha_size( mlt_frame self, int *size );
 extern int mlt_frame_get_audio( mlt_frame self, void **buffer, mlt_audio_format *format, int *frequency, int *channels, int *samples );
 extern int mlt_frame_set_audio( mlt_frame self, void *buffer, mlt_audio_format, int size, mlt_destructor );
 extern unsigned char *mlt_frame_get_waveform( mlt_frame self, int w, int h );
@@ -145,11 +141,7 @@ extern mlt_properties mlt_frame_get_unique_properties( mlt_frame self, mlt_servi
 extern mlt_frame mlt_frame_clone( mlt_frame self, int is_deep );
 
 /* convenience functions */
-extern const char * mlt_image_format_name( mlt_image_format format );
-extern int mlt_image_format_size( mlt_image_format format, int width, int height, int *bpp );
 extern void mlt_frame_write_ppm( mlt_frame frame );
-extern int mlt_image_format_planes( mlt_image_format format, int width, int height, void* data, unsigned char *planes[4], int strides[4]);
-extern mlt_image_format mlt_image_format_id( const char * name );
 
 /** This macro scales RGB into the YUV gamut - y is scaled by 219/255 and uv by 224/255. */
 #define RGB2YUV_601_SCALED(r, g, b, y, u, v)\
