@@ -3,7 +3,7 @@
  * \brief playlist service class
  * \see mlt_playlist_s
  *
- * Copyright (C) 2003-2021 Meltytech, LLC
+ * Copyright (C) 2003-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -1078,12 +1078,7 @@ int mlt_playlist_split( mlt_playlist self, int clip, mlt_position position )
 				mlt_properties split_properties = MLT_PRODUCER_PROPERTIES( split );
 				mlt_playlist_insert( self, split, clip + 1, 0, -1 );
 				mlt_properties_lock( entry_properties );
-				for ( i = 0; i < mlt_properties_count( entry_properties ); i ++ )
-				{
-					char *name = mlt_properties_get_name( entry_properties, i );
-					if ( name != NULL && !strncmp( name, "meta.", 5 ) )
-						mlt_properties_set( split_properties, name, mlt_properties_get_value( entry_properties, i ) );
-				}
+				mlt_properties_copy(split_properties, entry_properties, "meta.");
 				mlt_properties_unlock( entry_properties );
 				mlt_producer_close( split );
 			}
@@ -2087,7 +2082,7 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 	// Set the consumer progressive property
 	if ( progressive )
 	{
-		mlt_properties_set_int( properties, "consumer_deinterlace", progressive );
+		mlt_properties_set_int( properties, "consumer.progressive", progressive );
 		mlt_properties_set_int( properties, "test_audio", 1 );
 	}
 
