@@ -3,7 +3,7 @@
  * \brief abstraction for all producer services
  * \see mlt_producer_s
  *
- * Copyright (C) 2003-2019 Meltytech, LLC
+ * Copyright (C) 2003-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -38,7 +38,7 @@
 /* Forward references. */
 
 static int producer_get_frame( mlt_service self, mlt_frame_ptr frame, int index );
-static void mlt_producer_property_changed( mlt_service owner, mlt_producer self, char *name );
+static void mlt_producer_property_changed(mlt_service owner, mlt_producer self, mlt_event_data );
 static void mlt_producer_service_changed( mlt_service owner, mlt_producer self );
 
 /* for debugging */
@@ -108,7 +108,7 @@ int mlt_producer_init( mlt_producer self, void *child )
 
 			mlt_events_listen( properties, self, "service-changed", ( mlt_listener )mlt_producer_service_changed );
 			mlt_events_listen( properties, self, "property-changed", ( mlt_listener )mlt_producer_property_changed );
-			mlt_events_register( properties, "producer-changed", NULL );
+			mlt_events_register( properties, "producer-changed" );
 		}
 	}
 
@@ -125,10 +125,12 @@ int mlt_producer_init( mlt_producer self, void *child )
  * \param name the property that changed
  */
 
-static void mlt_producer_property_changed( mlt_service owner, mlt_producer self, char *name )
+static void mlt_producer_property_changed( mlt_service owner, mlt_producer self, mlt_event_data event_data)
 {
+	const char *name = mlt_event_data_to_string(event_data);
+	if (!name) return;
 	if ( !strcmp( name, "in" ) || !strcmp( name, "out" ) || !strcmp( name, "length" ) )
-		mlt_events_fire( MLT_PRODUCER_PROPERTIES( mlt_producer_cut_parent( self ) ), "producer-changed", NULL );
+		mlt_events_fire( MLT_PRODUCER_PROPERTIES( mlt_producer_cut_parent( self ) ), "producer-changed", mlt_event_data_none() );
 }
 
 /** Listener for service changes.
@@ -142,7 +144,7 @@ static void mlt_producer_property_changed( mlt_service owner, mlt_producer self,
 
 static void mlt_producer_service_changed( mlt_service owner, mlt_producer self )
 {
-	mlt_events_fire( MLT_PRODUCER_PROPERTIES( mlt_producer_cut_parent( self ) ), "producer-changed", NULL );
+	mlt_events_fire( MLT_PRODUCER_PROPERTIES( mlt_producer_cut_parent( self ) ), "producer-changed", mlt_event_data_none() );
 }
 
 /** Create and initialize a new producer.
@@ -310,6 +312,11 @@ mlt_properties mlt_producer_properties( mlt_producer self )
 
 int mlt_producer_seek( mlt_producer self, mlt_position position )
 {
+	if ( self->seek )
+	{
+		return self->seek( self, position );
+	}
+
 	// Determine eof handling
 	mlt_properties properties = MLT_PRODUCER_PROPERTIES( self );
 	char *eof = mlt_properties_get( properties, "eof" );
@@ -321,7 +328,11 @@ int mlt_producer_seek( mlt_producer self, mlt_position position )
 		mlt_producer_seek( mlt_producer_cut_parent( self ), position + mlt_producer_get_in( self ) );
 
 	// Check bounds
-	if ( position < 0 || mlt_producer_get_playtime( self ) == 0 )
+	if( mlt_service_identify( MLT_PRODUCER_SERVICE(self) ) == mlt_service_link_type )
+	{
+		// Do not bounds check a link.
+	}
+	else if ( position < 0 || mlt_producer_get_playtime( self ) == 0 )
 	{
 		position = 0;
 	}
@@ -454,6 +465,11 @@ double mlt_producer_get_fps( mlt_producer self )
 
 int mlt_producer_set_in_and_out( mlt_producer self, mlt_position in, mlt_position out )
 {
+	if ( self->set_in_and_out )
+	{
+		return self->set_in_and_out( self, in, out );
+	}
+
 	mlt_properties properties = MLT_PRODUCER_PROPERTIES( self );
 
 	// Correct ins and outs if necessary
@@ -716,19 +732,11 @@ static int producer_get_frame( mlt_service service, mlt_frame_ptr frame, int ind
 	// Pass on all meta properties from the producer/cut on to the frame
 	if ( *frame != NULL && self != NULL )
 	{
-		int i = 0;
 		mlt_properties p_props = MLT_PRODUCER_PROPERTIES( self );
 		mlt_properties f_props = MLT_FRAME_PROPERTIES( *frame );
 		mlt_properties_lock( p_props );
-		int count = mlt_properties_count( p_props );
-		for ( i = 0; i < count; i ++ )
-		{
-			char *name = mlt_properties_get_name( p_props, i );
-			if ( !strncmp( name, "meta.", 5 ) )
-				mlt_properties_set( f_props, name, mlt_properties_get_value( p_props, i ) );
-			else if ( !strncmp( name, "set.", 4 ) )
-				mlt_properties_set( f_props, name + 4, mlt_properties_get_value( p_props, i ) );
-		}
+		mlt_properties_copy(f_props, p_props, "meta.");
+		mlt_properties_pass(f_props, p_props, "set.");
 		mlt_properties_unlock( p_props );
 	}
 
@@ -932,7 +940,7 @@ static int on_start_producer( mlt_parser self, mlt_producer object )
 	mlt_properties properties = mlt_parser_properties( self );
 	mlt_properties producers = mlt_properties_get_data( properties, "producers", NULL );
 	mlt_producer parent = mlt_producer_cut_parent( object );
-	if ( mlt_service_identify( ( mlt_service )mlt_producer_cut_parent( object ) ) == producer_type && mlt_producer_is_cut( object ) )
+	if ( mlt_service_identify( ( mlt_service )mlt_producer_cut_parent( object ) ) == mlt_service_producer_type && mlt_producer_is_cut( object ) )
 	{
 		int ref_count = 0;
 		clip_references *old_refs = NULL;
