@@ -3,7 +3,7 @@
  * \brief video output definition
  * \see mlt_profile_s
  *
- * Copyright (C) 2007-2018 Meltytech, LLC
+ * Copyright (C) 2007-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -363,9 +363,17 @@ mlt_properties mlt_profile_list( )
 	if ( prefix == NULL )
 	{
 		prefix = mlt_environment( "MLT_DATA" );
-		filename = calloc( 1, strlen( prefix ) + strlen( PROFILES_DIR ) + 1 );
-		strcpy( filename, prefix );
-		strcat( filename, PROFILES_DIR );
+		if (prefix)
+		{
+			filename = calloc( 1, strlen( prefix ) + strlen( PROFILES_DIR ) + 1 );
+			strcpy( filename, prefix );
+			strcat( filename, PROFILES_DIR );
+		}
+		else
+		{			
+			filename = calloc( 1, strlen( PROFILES_DIR ) + 1 );
+			strcpy( filename, PROFILES_DIR );
+		}		
 		prefix = filename;
 	}
 
@@ -414,7 +422,7 @@ void mlt_profile_from_producer( mlt_profile profile, mlt_producer producer )
 	if ( ! mlt_service_get_frame( MLT_PRODUCER_SERVICE(producer), &fr, 0 ) && fr )
 	{
 		// Skip scaling and padding since not needed and we request mlt_image_none.
-		mlt_properties_set( MLT_FRAME_PROPERTIES(fr), "rescale.interp", "none" );
+		mlt_properties_set( MLT_FRAME_PROPERTIES(fr), "consumer.rescale", "none" );
 
 		if ( ! mlt_frame_get_image( fr, &buffer, &fmt, &w, &h, 0 ) )
 		{
@@ -446,9 +454,17 @@ void mlt_profile_from_producer( mlt_profile profile, mlt_producer producer )
 				profile->sample_aspect_num = mlt_properties_get_int( p, "meta.media.sample_aspect_num" );
 				profile->sample_aspect_den = mlt_properties_get_int( p, "meta.media.sample_aspect_den" );
 				profile->colorspace = mlt_properties_get_int( p, "meta.media.colorspace" );
-				profile->display_aspect_num = lrint( (double) profile->sample_aspect_num * profile->width
-					/ profile->sample_aspect_den );
-				profile->display_aspect_den = profile->height;
+				int n = profile->display_aspect_num = profile->sample_aspect_num * profile->width;
+				int m = profile->display_aspect_den = profile->sample_aspect_den * profile->height;
+				int gcd, remainder;
+				while (n) {
+					remainder = m % n;
+					m = n;
+					n = remainder;
+				}
+				gcd = m;
+				profile->display_aspect_num /= gcd;
+				profile->display_aspect_den /= gcd;
 				free( profile->description );
 				profile->description = strdup( "automatic" );
 				profile->is_explicit = 0;
