@@ -3,7 +3,7 @@
  * \brief Properties class declaration
  * \see mlt_properties_s
  *
- * Copyright (C) 2003-2020 Meltytech, LLC
+ * Copyright (C) 2003-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,6 +31,9 @@
  *
  * Properties is a combination list/dictionary of name/::mlt_property pairs.
  * It is also a base class for many of the other MLT classes.
+ *
+ * \event \em property-changed a property's value changed;
+ *   the event data is a string for the name of the property
  */
 
 struct mlt_properties_s
@@ -54,6 +57,7 @@ extern int mlt_properties_dec_ref( mlt_properties self );
 extern int mlt_properties_ref_count( mlt_properties self );
 extern void mlt_properties_mirror( mlt_properties self, mlt_properties that );
 extern int mlt_properties_inherit( mlt_properties self, mlt_properties that );
+extern int mlt_properties_copy( mlt_properties self, mlt_properties that, const char *prefix );
 extern int mlt_properties_pass( mlt_properties self, mlt_properties that, const char *prefix );
 extern void mlt_properties_pass_property( mlt_properties self, mlt_properties that, const char *name );
 extern int mlt_properties_pass_list( mlt_properties self, mlt_properties that, const char *list );
@@ -89,13 +93,16 @@ extern char *mlt_properties_serialise_yaml( mlt_properties self );
 extern void mlt_properties_lock( mlt_properties self );
 extern void mlt_properties_unlock( mlt_properties self );
 extern void mlt_properties_clear( mlt_properties self, const char *name );
+extern int mlt_properties_exists( mlt_properties self, const char *name );
 
 extern char *mlt_properties_get_time( mlt_properties, const char* name, mlt_time_format );
 extern char *mlt_properties_frames_to_time( mlt_properties, mlt_position, mlt_time_format );
 extern mlt_position mlt_properties_time_to_frames( mlt_properties, const char* time );
 
-extern mlt_color mlt_properties_get_color( mlt_properties, const char* name );
 extern int mlt_properties_set_color( mlt_properties, const char* name, mlt_color value );
+extern mlt_color mlt_properties_get_color( mlt_properties, const char* name );
+extern int mlt_properties_anim_set_color( mlt_properties self, const char *name, mlt_color value, int position, int length , mlt_keyframe_type keyframe_type );
+extern mlt_color mlt_properties_anim_get_color( mlt_properties self, const char *name, int position, int length );
 
 extern char* mlt_properties_anim_get( mlt_properties self, const char *name, int position, int length );
 extern int mlt_properties_anim_set( mlt_properties self, const char *name, const char *value, int position, int length );
@@ -104,6 +111,7 @@ extern int mlt_properties_anim_set_int( mlt_properties self, const char *name, i
 extern double mlt_properties_anim_get_double( mlt_properties self, const char *name, int position, int length );
 extern int mlt_properties_anim_set_double( mlt_properties self, const char *name, double value, int position, int length, mlt_keyframe_type keyframe_type );
 extern mlt_animation mlt_properties_get_animation( mlt_properties self, const char *name );
+extern int mlt_properties_is_anim(mlt_properties self, const char *name);
 
 extern int mlt_properties_set_rect( mlt_properties self, const char *name, mlt_rect value );
 extern mlt_rect mlt_properties_get_rect( mlt_properties self, const char *name );
@@ -112,4 +120,9 @@ extern mlt_rect mlt_properties_anim_get_rect( mlt_properties self, const char *n
 
 extern int mlt_properties_from_utf8( mlt_properties properties, const char *name_from, const char *name_to );
 extern int mlt_properties_to_utf8( mlt_properties properties, const char *name_from, const char *name_to );
+
+extern int mlt_properties_set_properties( mlt_properties self, const char *name, mlt_properties properties );
+extern mlt_properties mlt_properties_get_properties( mlt_properties self, const char *name );
+extern mlt_properties mlt_properties_get_properties_at( mlt_properties self, int index );
+
 #endif
