@@ -3,7 +3,7 @@
  * \brief sliced threading processing helper
  * \see mlt_slices_s
  *
- * Copyright (C) 2016-2017 Meltytech, LLC
+ * Copyright (C) 2016-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -135,15 +135,14 @@ static void* mlt_slices_worker( void* p )
 
 /** Initialize a sliced threading context
  *
- * \public \memberof mlt_slices_s
- * \deprecated
+ * \private \memberof mlt_slices_s
  * \param threads number of threads to use for job list, 0 for #cpus
  * \param policy scheduling policy of processing threads, -1 for normal
  * \param priority priority value that can be used with the scheduling algorithm, -1 for maximum
  * \return the context pointer
  */
 
-mlt_slices mlt_slices_init( int threads, int policy, int priority )
+static mlt_slices mlt_slices_init( int threads, int policy, int priority )
 {
 	pthread_attr_t tattr;
 	struct sched_param param;
@@ -219,12 +218,11 @@ mlt_slices mlt_slices_init( int threads, int policy, int priority )
 
 /** Destroy sliced threading context
  *
- * \public \memberof mlt_slices_s
- * \deprecated
+ * \private \memberof mlt_slices_s
  * \param ctx context pointer
  */
 
-void mlt_slices_close( mlt_slices ctx )
+static void mlt_slices_close( mlt_slices ctx )
 {
 	int j;
 
@@ -264,15 +262,20 @@ void mlt_slices_close( mlt_slices ctx )
 
 /** Run sliced execution
  *
- * \public \memberof mlt_slices_s
- * \deprecated
+ * \private \memberof mlt_slices_s
  * \param ctx context pointer
  * \param jobs number of jobs to process
- * \param proc number of jobs to process
+ * \param proc a pointer to the function that will be called
+ * \param cookie an opaque data pointer passed to \p proc
  */
 
-void mlt_slices_run( mlt_slices ctx, int jobs, mlt_slices_proc proc, void* cookie )
+static void mlt_slices_run( mlt_slices ctx, int jobs, mlt_slices_proc proc, void* cookie )
 {
+	if ( jobs == 1 )
+	{
+		proc(0, 0, 1, cookie);
+		return;
+	}
 	struct mlt_slices_runtime_s runtime, *r = &runtime;
 
 	/* lock */
@@ -411,3 +414,27 @@ void mlt_slices_run_fifo(int jobs, mlt_slices_proc proc, void *cookie)
 	return mlt_slices_run( mlt_slices_get_global( mlt_policy_fifo ),
 	   jobs, proc, cookie );
 }
+
+
+/** Compute size of a slice.
+ *
+ * This a helper function for use in a mlt_slices_proc() to get the number of
+ * pixels over which to operate.
+ *
+ * \public \memberof mlt_slices_s
+ * \param jobs the number of slices
+ * \param index the zero-based index of the current slice
+ * \param input_size the size of a dimension, usually in pixel units, for example height
+ * \param[out] start the optional starting unit for this slice
+ * \return the size of the slice, typically in pixel units
+ */
+
+int mlt_slices_size_slice(int jobs, int index, int input_size, int *start)
+{
+	int size = (input_size + jobs - 1) / jobs;
+	int my_start = index * size;
+	if (start) {
+		*start = my_start;
+	}
+	return CLAMP(input_size - my_start, 0, size);
+}
