@@ -1,7 +1,7 @@
 /**
  * MltFilteredConsumer.h - MLT Wrapper
- * Copyright (C) 2004-2005 Charles Yates
- * Author: Charles Yates <charles.yates@pandora.be>
+ * Copyright (C) 2004-2015 Meltytech, LLC
+ * Author: Charles Yates <charles.yates@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -15,13 +15,13 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef _MLTPP_FILTERED_CONSUMER_H_
-#define _MLTPP_FILTERED_CONSUMER_H_
+#ifndef MLTPP_FILTERED_CONSUMER_H
+#define MLTPP_FILTERED_CONSUMER_H
 
-#include "config.h"
+#include "MltConfig.h"
 
 #include "MltConsumer.h"
 #include "MltFilter.h"
@@ -42,7 +42,7 @@ namespace Mlt
 			FilteredConsumer( Profile& profile, const char *id, const char *arg = NULL );
 			FilteredConsumer( Consumer &consumer );
 			virtual ~FilteredConsumer( );
-			int connect( Service &service );
+			int connect( Service &service ) override;
 			int attach( Filter &filter );
 			int last( Filter &filter );
 			int detach( Filter &filter );
