@@ -199,17 +199,13 @@ static int convert_image( mlt_frame frame, uint8_t **image, mlt_image_format *fo
 		*image = output;
 		*format = output_format;
 		mlt_frame_set_image( frame, output, size, mlt_pool_release );
-		mlt_properties_set_int( properties, "format", output_format );
-		mlt_properties_set_int(properties, "width", out_width);
-		mlt_properties_set_int(properties, "height", out_height);
 
 		if (out_width == width && out_height == height)
 		if ( output_format == mlt_image_rgba )
 		{
 			register int len = width * height;
 			int alpha_size = 0;
-			uint8_t *alpha = mlt_frame_get_alpha( frame );
-			mlt_properties_get_data( properties, "alpha", &alpha_size );
+			uint8_t *alpha = mlt_frame_get_alpha_size( frame, &alpha_size );
 
 			if ( alpha && alpha_size >= len )
 			{
@@ -233,6 +229,10 @@ static int convert_image( mlt_frame frame, uint8_t **image, mlt_image_format *fo
 				}
 			}
 		}
+
+		mlt_properties_set_int( properties, "format", output_format );
+		mlt_properties_set_int(properties, "width", out_width);
+		mlt_properties_set_int(properties, "height", out_height);
 	}
 	return error;
 }
