@@ -1,6 +1,6 @@
 /*
  * filter_crop.c -- cropping filter
- * Copyright (C) 2009-2021 Meltytech, LLC
+ * Copyright (C) 2009-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -112,9 +112,8 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 		}
 
 		// We should resize the alpha too
-		uint8_t *alpha = mlt_frame_get_alpha( frame );
 		int alpha_size = 0;
-		mlt_properties_get_data( properties, "alpha", &alpha_size );
+		uint8_t *alpha = mlt_frame_get_alpha_size(frame, &alpha_size);
 		if ( alpha && alpha_size >= ( *width * *height ) )
 		{
 			uint8_t *newalpha = mlt_pool_alloc( owidth * oheight );
