@@ -78,29 +78,22 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 	mlt_image_format current_format = mlt_properties_get_int( producer_props, "_format" );
 
 	// Parse the colour
-	if ( now && strchr( now, '/' ) )
-	{
-		now = strdup( strrchr( now, '/' ) + 1 );
-		mlt_properties_set( producer_props, "resource", now );
-		free( now );
-		now = mlt_properties_get( producer_props, "resource" );
-	}
 	mlt_color color = mlt_properties_get_color( producer_props, "resource" );
 
 	if ( mlt_properties_get( producer_props, "mlt_image_format") )
 		*format = mlt_image_format_id( mlt_properties_get( producer_props, "mlt_image_format") );
 
 	// Choose suitable out values if nothing specific requested
-	if ( *format == mlt_image_none || *format == mlt_image_glsl )
-		*format = mlt_image_rgb24a;
+	if ( *format == mlt_image_none || *format == mlt_image_movit )
+		*format = mlt_image_rgba;
 	if ( *width <= 0 )
 		*width = mlt_service_profile( MLT_PRODUCER_SERVICE(producer) )->width;
 	if ( *height <= 0 )
 		*height = mlt_service_profile( MLT_PRODUCER_SERVICE(producer) )->height;
 	
 	// Choose default image format if specific request is unsupported
-	if (*format!=mlt_image_yuv420p  && *format!=mlt_image_yuv422  && *format!=mlt_image_rgb24 && *format!= mlt_image_glsl && *format!= mlt_image_glsl_texture)
-		*format = mlt_image_rgb24a;
+	if (*format!=mlt_image_yuv420p  && *format!=mlt_image_yuv422  && *format!=mlt_image_rgb && *format!= mlt_image_movit && *format!= mlt_image_opengl_texture)
+		*format = mlt_image_rgba;
 
 	// See if we need to regenerate
 	if ( !now || ( then && strcmp( now, then ) ) || *width != current_width || *height != current_height || *format != current_format )
@@ -163,7 +156,7 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 			mlt_properties_set_int( properties, "colorspace", 601 );
 			break;
 		}
-		case mlt_image_rgb24:
+		case mlt_image_rgb:
 			while ( --i )
 			{
 				*p ++ = color.r;
@@ -171,11 +164,11 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 				*p ++ = color.b;
 			}
 			break;
-		case mlt_image_glsl:
-		case mlt_image_glsl_texture:
+		case mlt_image_movit:
+		case mlt_image_opengl_texture:
 			memset(p, 0, size);
 			break;
-		case mlt_image_rgb24a:
+		case mlt_image_rgba:
 			while ( --i )
 			{
 				*p ++ = color.r;
@@ -199,7 +192,7 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 	uint8_t *alpha = NULL;
 
 	// Initialise the alpha
-	if (color.a < 255 || *format == mlt_image_rgb24a) {
+	if (color.a < 255 || *format == mlt_image_rgba) {
 		alpha_size = *width * *height;
 		alpha = mlt_pool_alloc( alpha_size );
 		if ( alpha )
@@ -251,7 +244,26 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 		// colour is an alias for resource
 		if ( mlt_properties_get( producer_props, "colour" ) != NULL )
 			mlt_properties_set( producer_props, "resource", mlt_properties_get( producer_props, "colour" ) );
-		
+
+		char *colorstring = mlt_properties_get( producer_props, "resource" );
+		if ( colorstring && strchr( colorstring, '/' ) )
+		{
+			colorstring = strdup( strrchr( colorstring, '/' ) + 1 );
+			mlt_properties_set( producer_props, "resource", colorstring );
+			free( colorstring );
+		}
+
+		// Check if we have a predefined image format
+		if ( mlt_properties_exists( producer_props, "mlt_image_format") )
+		{
+			int image_format = mlt_image_format_id( mlt_properties_get( producer_props, "mlt_image_format") );
+			mlt_properties_set_int( properties, "format", image_format );
+		} else {
+			mlt_color color = mlt_properties_get_color( producer_props, "resource" );
+			// Inform framework of the default frame format for this producer
+			mlt_properties_set_int( properties, "format", color.a < 255 ? mlt_image_rgba : mlt_image_yuv422 );
+		}
+
 		// Push the get_image method
 		mlt_frame_push_service( *frame, producer );
 		mlt_frame_push_get_image( *frame, producer_get_image );
