@@ -1,6 +1,6 @@
 /*
  * producer_consumer.c -- produce as a consumer of an encapsulated producer
- * Copyright (C) 2008-2021 Meltytech, LLC
+ * Copyright (C) 2008-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -60,7 +60,7 @@ static int get_image( mlt_frame frame, uint8_t **image, mlt_image_format *format
 	*image = new_image;
 	
 	// Copy the alpha channel
-	uint8_t *alpha = mlt_properties_get_data( MLT_FRAME_PROPERTIES( nested_frame ), "alpha", &size );
+	uint8_t *alpha = mlt_frame_get_alpha_size(nested_frame, &size);
 	if ( alpha && size > 0 )
 	{
 		new_image = mlt_pool_alloc( size );
