@@ -1,6 +1,6 @@
 /*
  * transition_luma.c -- a generic dissolve/wipe processor
- * Copyright (C) 2003-2020 Meltytech, LLC
+ * Copyright (C) 2003-2022 Meltytech, LLC
  *
  * Adapted from Kino Plugin Timfx, which is
  * Copyright (C) 2002 Timothy M. Shead <tshead@k-3d.com>
@@ -85,14 +85,13 @@ static int dissolve_slice( int id, int index, int count, void *context )
 {
 	struct dissolve_slice_context ctx = *((struct dissolve_slice_context*) context);
 	int stride = ctx.width * 2;
-	int slice_height = (ctx.height + count - 1) / count;
+	int slice_start, slice_height = mlt_slices_size_slice(count, index, ctx.height, &slice_start);
 	int i;
 
-	ctx.dst_image += index * slice_height * stride;
-	ctx.src_image += index * slice_height * stride;
-	if (ctx.dst_alpha) ctx.dst_alpha += index * slice_height * ctx.width;
-	if (ctx.src_alpha) ctx.src_alpha += index * slice_height * ctx.width;
-	slice_height = MIN(slice_height, ctx.height - index * slice_height);
+	ctx.dst_image += slice_start * stride;
+	ctx.src_image += slice_start * stride;
+	if (ctx.dst_alpha) ctx.dst_alpha += slice_start * ctx.width;
+	if (ctx.src_alpha) ctx.src_alpha += slice_start * ctx.width;
 
 	for (i = 0; i < slice_height; i++) {
 		composite_line_yuv_float( ctx.dst_image, ctx.src_image, ctx.width, ctx.src_alpha, ctx.dst_alpha, ctx.weight );
@@ -118,9 +117,9 @@ static inline int dissolve_yuv( mlt_frame frame, mlt_frame that, float weight, i
 	if ( mlt_properties_get( &frame->parent, "distort" ) )
 		mlt_properties_set( &that->parent, "distort", mlt_properties_get( &frame->parent, "distort" ) );
 	mlt_frame_get_image( frame, &p_dest, &format, &width, &height, 1 );
-	alpha_dst = mlt_frame_get_alpha_mask( frame );
+	alpha_dst = mlt_frame_get_alpha( frame );
 	mlt_frame_get_image( that, &p_src, &format, &width_src, &height_src, 0 );
-	alpha_src = mlt_frame_get_alpha_mask( that );
+	alpha_src = mlt_frame_get_alpha( that );
 	int is_translucent = ( alpha_dst && !is_opaque(alpha_dst, width, height) )
 	                  || ( alpha_src && !is_opaque(alpha_src, width_src, height_src) );
 
@@ -148,8 +147,8 @@ static inline int dissolve_yuv( mlt_frame frame, mlt_frame that, float weight, i
 			composite_line_yuv( p_dest, p_src, width_src, alpha_src, alpha_dst, mix, NULL, 0, 0 );
 			p_src += width_src << 1;
 			p_dest += width << 1;
-			alpha_src += width_src;
-			alpha_dst += width;
+			if ( alpha_src ) alpha_src += width_src;
+			if ( alpha_dst ) alpha_dst += width;
 		}
 	}
 
@@ -205,9 +204,9 @@ static void luma_composite( mlt_frame a_frame, mlt_frame b_frame, int luma_width
 	if ( mlt_properties_get( &a_frame->parent, "distort" ) )
 		mlt_properties_set( &b_frame->parent, "distort", mlt_properties_get( &a_frame->parent, "distort" ) );
 	mlt_frame_get_image( a_frame, &p_dest, &format_dest, &width_dest, &height_dest, 1 );
-	alpha_dest = mlt_frame_get_alpha_mask( a_frame );
+	alpha_dest = mlt_frame_get_alpha( a_frame );
 	mlt_frame_get_image( b_frame, &p_src, &format_src, &width_src, &height_src, 0 );
-	alpha_src = mlt_frame_get_alpha_mask( b_frame );
+	alpha_src = mlt_frame_get_alpha( b_frame );
 
 	if ( *width == 0 || *height == 0 )
 		return;
@@ -471,7 +470,7 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 					if (luma_image) {
 						if (is_clip) {
 							yuv422_to_luma16(luma_image, &luma_bitmap, luma_width, luma_height,
-								mlt_properties_get_int(MLT_FRAME_PROPERTIES(luma_frame), "full_luma"));
+								mlt_properties_get_int(MLT_FRAME_PROPERTIES(luma_frame), "full_range"));
 						} else {
 							mlt_luma_map_from_yuv422(luma_image, &luma_bitmap, luma_width, luma_height);
 						}
@@ -491,6 +490,7 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 					mlt_properties_set_data(properties, "producer", producer, 0, (mlt_destructor) mlt_producer_close, NULL);
 				} else {
 					// Cleanup the luma producer
+					mlt_properties_clear(properties, "producer");
 					mlt_producer_close(producer);
 					producer = NULL;
 				}
@@ -503,7 +503,7 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 	float frame_delta = mlt_transition_get_progress_delta( transition, a_frame );
 	float luma_softness = mlt_properties_get_double( properties, "softness" );
 	int progressive = 
-			mlt_properties_get_int( a_props, "consumer_deinterlace" ) ||
+			mlt_properties_get_int( a_props, "consumer.progressive" ) ||
 			mlt_properties_get_int( properties, "progressive" ) ||
 			mlt_properties_get_int( b_props, "luma.progressive" );
 	int top_field_first =  mlt_properties_get_int( b_props, "top_field_first" );
@@ -519,13 +519,13 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 	if ( mlt_properties_get( properties, "fixed" ) )
 		mix = mlt_properties_get_double( properties, "fixed" );
 
-	mlt_service_unlock( MLT_TRANSITION_SERVICE( transition ) );
-
-	if (producer)
-	{
+	if (producer) {
 		invert = !invert;
 		mix = 0.5f;
+	} else {
+		mlt_service_unlock( MLT_TRANSITION_SERVICE( transition ) );
 	}
+
 	if ( luma_width > 0 && luma_height > 0 && luma_bitmap != NULL )
 	{
 		reverse = invert ? !reverse : reverse;
@@ -542,6 +542,9 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 		// Dissolve the frames using the time offset for mix value
 		dissolve_yuv( a_frame, b_frame, mix, *width, *height, threads, alpha_over );
 	}
+	if (producer) {
+		mlt_service_unlock( MLT_TRANSITION_SERVICE( transition ) );
+	}
 
 	// Extract the a_frame image info
 	*width = mlt_properties_get_int( !invert ? a_props : b_props, "width" );
