@@ -1,6 +1,6 @@
 /*
  * producer_decklink.c -- input from Blackmagic Design DeckLink
- * Copyright (C) 2011-2019 Meltytech, LLC
+ * Copyright (C) 2011-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -47,9 +47,7 @@ static int copy_lines_sliced_proc( int id, int idx, int jobs, void* cookie )
 	int c, H, Y, i;
 	struct copy_lines_sliced_desc *ctx = (struct copy_lines_sliced_desc*)cookie;
 
-	H = ( ctx->h + jobs ) / jobs;
-	Y = idx * H;
-	H = MIN( H, ctx->h - Y );
+	H = mlt_slices_size_slice(jobs, idx, ctx->h, &Y);
 
 	if ( ctx->in_fmt == bmdFormat10BitYUV ) // bmdFormat10BitYUV -> mlt_image_yuv422p16
 	{
@@ -793,8 +791,9 @@ static void producer_close( mlt_producer producer )
 extern "C" {
 
 // Listen for the list_devices property to be set
-static void on_property_changed( void*, mlt_properties properties, const char *name )
+static void on_property_changed( void*, mlt_properties properties, mlt_event_data event_data )
 {
+	const char *name = mlt_event_data_to_string(event_data);
 	IDeckLinkIterator* decklinkIterator = NULL;
 	IDeckLink* decklink = NULL;
 	IDeckLinkInput* decklinkInput = NULL;
