@@ -1,7 +1,7 @@
 /*
  * factory.c -- the factory method interfaces
  * Copyright (c) 2008 Marco Gittler <g.marco@freenet.de>
- * Copyright (C) 2009-2020 Meltytech, LLC
+ * Copyright (C) 2009-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,19 +33,22 @@
 #include <math.h>
 
 
-#if defined(_WIN32)
-#define LIBSUF ".dll"
-#define FREI0R_PLUGIN_PATH "\\lib\\frei0r-1"
-#elif defined(__APPLE__) && defined(RELOCATABLE)
-#define LIBSUF ".so"
-#define FREI0R_PLUGIN_PATH "/PlugIns/frei0r-1"
+#ifdef _WIN32
+#  define LIBSUF ".dll"
+#  define FREI0R_PLUGIN_PATH "\\lib\\frei0r-1"
 #else
-#define LIBSUF ".so"
-#define FREI0R_PLUGIN_PATH "/usr/lib/frei0r-1:/usr/lib64/frei0r-1:/opt/local/lib/frei0r-1:/usr/local/lib/frei0r-1:$HOME/.frei0r-1/lib"
+#  define LIBSUF ".so"
+#  ifdef RELOCATABLE
+#    ifdef __APPLE__
+#      define FREI0R_PLUGIN_PATH "/PlugIns/frei0r-1"
+#    else
+#      define FREI0R_PLUGIN_PATH "/lib/frei0r-1"
+#    endif
+#  else
+#    define FREI0R_PLUGIN_PATH "/usr/lib/frei0r-1:/usr/lib64/frei0r-1:/opt/local/lib/frei0r-1:/usr/local/lib/frei0r-1:$HOME/.frei0r-1/lib"
+#  endif
 #endif
 
-#define GET_FREI0R_PATH (getenv("FREI0R_PATH") ? getenv("FREI0R_PATH") : getenv("MLT_FREI0R_PLUGIN_PATH") ? getenv("MLT_FREI0R_PLUGIN_PATH") : FREI0R_PLUGIN_PATH)
-
 extern mlt_filter filter_frei0r_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_cairoblend_mode_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_frame filter_process( mlt_filter filter, mlt_frame frame );
@@ -57,18 +60,13 @@ extern mlt_frame transition_process( mlt_transition transition, mlt_frame a_fram
 
 static char* get_frei0r_path()
 {
-#ifdef _WIN32
-	char *dirname = malloc( strlen( mlt_environment( "MLT_APPDIR" ) ) + strlen( FREI0R_PLUGIN_PATH ) + 1 );
-	strcpy( dirname, mlt_environment( "MLT_APPDIR" ) );
-	strcat( dirname, FREI0R_PLUGIN_PATH );
-	return dirname;
-#elif defined(__APPLE__) && defined(RELOCATABLE)
+#if defined(_WIN32) || defined(RELOCATABLE)
 	char *dirname = malloc( strlen( mlt_environment( "MLT_APPDIR" ) ) + strlen( FREI0R_PLUGIN_PATH ) + 1 );
 	strcpy( dirname, mlt_environment( "MLT_APPDIR" ) );
 	strcat( dirname, FREI0R_PLUGIN_PATH );
 	return dirname;
 #else
-	return strdup( GET_FREI0R_PATH );
+	return strdup(getenv("FREI0R_PATH") ? getenv("FREI0R_PATH") : getenv("MLT_FREI0R_PLUGIN_PATH") ? getenv("MLT_FREI0R_PLUGIN_PATH") : FREI0R_PLUGIN_PATH);
 #endif
 }
 
@@ -100,13 +98,13 @@ static mlt_properties fill_param_info(mlt_service_type type, const char *service
 	struct stat stat_buff;
 
 	switch (type) {
-		case producer_type:
+		case mlt_service_producer_type:
 			strcpy(servicetype, "producer");
 			break;
-		case filter_type:
+		case mlt_service_filter_type:
 			strcpy(servicetype, "filter");
 			break;
-		case transition_type:
+		case mlt_service_transition_type:
 			strcpy(servicetype, "transition");
 			break;
 		default:
@@ -150,7 +148,7 @@ static mlt_properties fill_param_info(mlt_service_type type, const char *service
 	}
 	plginfo(&info);
 	snprintf(string, sizeof(string) , "%d" , info.minor_version);
-	mlt_properties_set_double(metadata, "schema_version", 0.3);
+	mlt_properties_set(metadata, "schema_version", "7.0");
 	mlt_properties_set(metadata, "title" , info.name);
 	mlt_properties_set_double(metadata, "version",
 		info.major_version +  info.minor_version / pow(10, strlen(string)));
@@ -158,13 +156,13 @@ static mlt_properties fill_param_info(mlt_service_type type, const char *service
 	mlt_properties_set(metadata, "description" , info.explanation);
 	mlt_properties_set(metadata, "creator" , info.author);
 	switch (type) {
-		case producer_type:
+		case mlt_service_producer_type:
 			mlt_properties_set(metadata, "type", "producer");
 			break;
-		case filter_type:
+		case mlt_service_filter_type:
 			mlt_properties_set(metadata, "type", "filter");
 			break;
-		case transition_type:
+		case mlt_service_transition_type:
 			mlt_properties_set(metadata, "type", "transition");
 			break;
 		default:
@@ -195,6 +193,7 @@ static mlt_properties fill_param_info(mlt_service_type type, const char *service
 			f0r_get_param_value(instance, &deflt, j);
 			mlt_properties_set_double(pnum, "default", CLAMP(deflt, 0.0, 1.0));
 			mlt_properties_set(pnum, "mutable", "yes" );
+			mlt_properties_set(pnum, "animation", "yes" );
 			mlt_properties_set(pnum, "widget", "spinner");
 		} else if (paraminfo.type == F0R_PARAM_BOOL) {
 			double deflt = 0;
@@ -204,6 +203,7 @@ static mlt_properties fill_param_info(mlt_service_type type, const char *service
 			f0r_get_param_value(instance, &deflt, j);
 			mlt_properties_set_int(pnum, "default", deflt != 0.0);
 			mlt_properties_set(pnum, "mutable", "yes");
+			mlt_properties_set(pnum, "animation", "yes" );
 			mlt_properties_set(pnum, "widget", "checkbox");
 		} else if (paraminfo.type == F0R_PARAM_COLOR) {
 			char colorstr[8];
@@ -216,6 +216,7 @@ static mlt_properties fill_param_info(mlt_service_type type, const char *service
 			colorstr[7] = 0;
 			mlt_properties_set(pnum, "default", colorstr);
 			mlt_properties_set(pnum, "mutable", "yes");
+			mlt_properties_set(pnum, "animation", "yes" );
 			mlt_properties_set(pnum, "widget", "color");
 		} else if (paraminfo.type == F0R_PARAM_STRING) {
 			char *deflt = NULL;
@@ -265,7 +266,7 @@ static void* load_lib(mlt_profile profile, mlt_service_type type , void* handle,
 		mlt_properties properties = NULL;
 		char minor[12];
 
-		if (type == producer_type && info.plugin_type == F0R_PLUGIN_TYPE_SOURCE) {
+		if (type == mlt_service_producer_type && info.plugin_type == F0R_PLUGIN_TYPE_SOURCE) {
 			mlt_producer producer = mlt_producer_new(profile);
 			if (producer)
 			{
@@ -281,7 +282,7 @@ static void* load_lib(mlt_profile profile, mlt_service_type type , void* handle,
 
 				ret = producer;
 			}
-		} else if (type == filter_type && info.plugin_type == F0R_PLUGIN_TYPE_FILTER) {
+		} else if (type == mlt_service_filter_type && info.plugin_type == F0R_PLUGIN_TYPE_FILTER) {
 			mlt_filter filter = mlt_filter_new();
 			if (filter)
 			{
@@ -297,7 +298,7 @@ static void* load_lib(mlt_profile profile, mlt_service_type type , void* handle,
 
 				ret = filter;
 			}
-		} else if (type == transition_type && info.plugin_type == F0R_PLUGIN_TYPE_MIXER2) {
+		} else if (type == mlt_service_transition_type && info.plugin_type == F0R_PLUGIN_TYPE_MIXER2) {
 			mlt_transition transition = mlt_transition_new();
 			if (transition)
 			{
@@ -469,8 +470,8 @@ MLT_REPOSITORY
 							dlclose(handle);
 							continue;
 						}
-						MLT_REGISTER(producer_type, pluginname, create_frei0r_item);
-						MLT_REGISTER_METADATA(producer_type, pluginname, fill_param_info, name);
+						MLT_REGISTER(mlt_service_producer_type, pluginname, create_frei0r_item);
+						MLT_REGISTER_METADATA(mlt_service_producer_type, pluginname, fill_param_info, name);
 					}
 					else if (firstname && info.plugin_type == F0R_PLUGIN_TYPE_FILTER) {
 						if (mlt_properties_get(mlt_repository_filters(repository), pluginname))
@@ -478,8 +479,8 @@ MLT_REPOSITORY
 							dlclose(handle);
 							continue;
 						}
-						MLT_REGISTER(filter_type, pluginname, create_frei0r_item);
-						MLT_REGISTER_METADATA(filter_type, pluginname, fill_param_info, name);
+						MLT_REGISTER(mlt_service_filter_type, pluginname, create_frei0r_item);
+						MLT_REGISTER_METADATA(mlt_service_filter_type, pluginname, fill_param_info, name);
 					}
 					else if (firstname && info.plugin_type==F0R_PLUGIN_TYPE_MIXER2 ) {
 						if (mlt_properties_get(mlt_repository_transitions(repository), pluginname))
@@ -487,8 +488,8 @@ MLT_REPOSITORY
 							dlclose(handle);
 							continue;
 						}
-						MLT_REGISTER(transition_type, pluginname, create_frei0r_item);
-						MLT_REGISTER_METADATA(transition_type, pluginname, fill_param_info, name);
+						MLT_REGISTER(mlt_service_transition_type, pluginname, create_frei0r_item);
+						MLT_REGISTER_METADATA(mlt_service_transition_type, pluginname, fill_param_info, name);
 					}
 				}
 				dlclose(handle);
@@ -499,6 +500,6 @@ MLT_REPOSITORY
 	mlt_tokeniser_close(tokeniser);
 	mlt_properties_close(blacklist);
 	free(frei0r_path);
-	MLT_REGISTER(filter_type, "cairoblend_mode", filter_cairoblend_mode_init);
-	MLT_REGISTER_METADATA(filter_type, "cairoblend_mode", metadata, "filter_cairoblend_mode.yml");
+	MLT_REGISTER(mlt_service_filter_type, "cairoblend_mode", filter_cairoblend_mode_init);
+	MLT_REGISTER_METADATA(mlt_service_filter_type, "cairoblend_mode", metadata, "filter_cairoblend_mode.yml");
 }
