@@ -1,7 +1,7 @@
 /*
  * frei0r_helper.c -- frei0r helper
  * Copyright (c) 2008 Marco Gittler <g.marco@freenet.de>
- * Copyright (C) 2009-2020 Meltytech, LLC
+ * Copyright (C) 2009-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -78,6 +78,20 @@ static int f0r_update2_slice( int id, int index, int count, void *context )
 	return 0;
 }
 
+static void destruct_ctor(mlt_properties prop)
+{
+	void (*f0r_destruct) (f0r_instance_t instance) = mlt_properties_get_data(prop, "f0r_destruct", NULL);
+	for (int i = 0; i < mlt_properties_count(prop); i++) {
+		if (strstr(mlt_properties_get_name(prop, i), "ctor-")) {
+			void *inst = mlt_properties_get_data(prop, mlt_properties_get_name(prop, i), NULL);
+			if (inst) {
+				f0r_destruct((f0r_instance_t) inst);
+				mlt_properties_clear(prop, mlt_properties_get_name(prop, i));
+			}
+		}
+	}
+}
+
 int process_frei0r_item( mlt_service service, mlt_position position, double time,
 	int length, mlt_frame frame, uint8_t **image, int *width, int *height )
 {
@@ -136,6 +150,8 @@ int process_frei0r_item( mlt_service service, mlt_position position, double time
 
 	f0r_instance_t inst = mlt_properties_get_data(prop, ctorname, NULL);
 	if (!inst) {
+		if (slice_count > 1)
+			destruct_ctor(prop);
 		inst = f0r_construct(*width, slice_height);
 		mlt_properties_set_data(prop, ctorname, inst, 0, NULL, NULL);
 	}
@@ -208,7 +224,7 @@ int process_frei0r_item( mlt_service service, mlt_position position, double time
 					{
 						f0r_param_color_t f_color;
 						mlt_color m_color = mlt_properties_get(prop, index) ?
-							mlt_properties_get_color(prop, index) : mlt_properties_get_color(prop, pinfo.name);
+							mlt_properties_anim_get_color(prop, index, position, length) : mlt_properties_anim_get_color(prop, pinfo.name, position, length);
 						f_color.r = (float) m_color.r / 255.0f;
 						f_color.g = (float) m_color.g / 255.0f;
 						f_color.b = (float) m_color.b / 255.0f;
@@ -233,20 +249,20 @@ int process_frei0r_item( mlt_service service, mlt_position position, double time
 	uint32_t *dest = result;
 
 	if (info.color_model == F0R_COLOR_MODEL_BGRA8888) {
-		if (type == producer_type) {
+		if (type == mlt_service_producer_type) {
 			dest = source[0];
 		} else {
 			rgba_bgra(image[0], (uint8_t*) result, *width, *height);
 			source[0] = result;
 			dest = (uint32_t*) image[0];
-			if (type == transition_type && f0r_update2) {
+			if (type == mlt_service_transition_type && f0r_update2) {
 				extra = mlt_pool_alloc(video_area * sizeof(uint32_t));
 				rgba_bgra(image[1], (uint8_t*) extra, *width, *height);
 				source[1] = extra;
 			}
 		}
 	}
-	if (type == producer_type) {
+	if (type == mlt_service_producer_type) {
 		if (slice_count > 0) {
 			struct update_context ctx = {
 				.frei0r = inst,
@@ -261,7 +277,7 @@ int process_frei0r_item( mlt_service service, mlt_position position, double time
 		} else {
 			f0r_update(inst, time, NULL, dest);
 		}
-	} else if (type == filter_type) {
+	} else if (type == mlt_service_filter_type) {
 		if (slice_count > 0) {
 			struct update_context ctx = {
 				.frei0r = inst,
@@ -276,7 +292,7 @@ int process_frei0r_item( mlt_service service, mlt_position position, double time
 		} else {
 			f0r_update(inst, time, source[0], dest);
 		}
-	} else if (type == transition_type && f0r_update2) {
+	} else if (type == mlt_service_transition_type && f0r_update2) {
 		if (slice_count > 0) {
 			struct update_context ctx = {
 				.frei0r = inst,
