@@ -9,6 +9,7 @@
 #include "model/animation/frame_time.hpp"
 #include "model/animation/join_animatables.hpp"
 #include "math/bezier/bezier.hpp"
+#include "app/utils/string_view.hpp"
 
 #include "path_parser.hpp"
 #include "detail.hpp"
@@ -23,7 +24,7 @@ public:
     public:
         enum Type
         {
-            Vector, Bezier
+            Vector, Bezier, String
         };
 
         ValueVariant(std::vector<qreal> v = {})
@@ -34,6 +35,10 @@ public:
             : value_(std::move(v))
         {}
 
+        ValueVariant(QString v )
+            : value_(std::move(v))
+        {}
+
         Type type() const { return Type(value_.index()); }
 
         const std::vector<qreal>& vector() const
@@ -46,6 +51,11 @@ public:
             return std::get<int(Type::Bezier)>(value_);
         }
 
+        const QString& string() const
+        {
+            return std::get<int(Type::String)>(value_);
+        }
+
         ValueVariant lerp(const ValueVariant& other, qreal t) const
         {
             if ( type() != other.type() )
@@ -63,6 +73,9 @@ public:
                         return mb;
                     }
                     return *this;
+                case Type::String:
+                    return t < 1 ? string() : other.string();
+
             }
 
             return {};
@@ -80,7 +93,7 @@ public:
         }
 
     private:
-        std::variant<std::vector<qreal>, math::bezier::MultiBezier> value_;
+        std::variant<std::vector<qreal>, math::bezier::MultiBezier, QString> value_;
     };
 
     struct PropertyKeyframe
@@ -128,7 +141,7 @@ public:
         std::map<QString, AnimatedProperty> properties;
         QDomElement element;
 
-        bool has(const QString& name)
+        bool has(const QString& name) const
         {
             return properties.count(name);
         }
@@ -246,7 +259,7 @@ public:
             return {};
         }
 
-        auto split = v.splitRef(separator,
+        auto split = ::utils::split_ref(v, separator,
 #if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
         Qt::SkipEmptyParts
 #else
@@ -275,13 +288,13 @@ public:
             {"h", hours}
         };
 
-        if ( !match.capturedRef("unit").isEmpty() )
-            return match.capturedRef("timecount").toDouble() * units.at(match.captured("unit")) * fps;
+        if ( !match.captured("unit").isEmpty() )
+            return match.captured("timecount").toDouble() * units.at(match.captured("unit")) * fps;
 
         return (
-            match.capturedRef("hours").toDouble() * hours +
-            match.capturedRef("minutes").toDouble() * minutes +
-            match.capturedRef("seconds").toDouble()
+            match.captured("hours").toDouble() * hours +
+            match.captured("minutes").toDouble() * minutes +
+            match.captured("seconds").toDouble()
         ) * fps;
     }
 
@@ -293,6 +306,8 @@ public:
                 return split_values(str);
             case ValueVariant::Bezier:
                 return PathDParser(str).parse();
+            case ValueVariant::String:
+                return str;
         }
 
         return {};
@@ -304,6 +319,8 @@ public:
         ValueVariant::Type type = ValueVariant::Vector;
         if ( attr == "d" )
             type = ValueVariant::Bezier;
+        else if ( attr == "display" )
+            type = ValueVariant::String;
 
         std::vector<ValueVariant> values;
 
