@@ -1,6 +1,6 @@
 /*
  * consumer_jack.c -- a JACK audio consumer
- * Copyright (C) 2011-2020 Meltytech, LLC
+ * Copyright (C) 2011-2021 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -64,7 +64,7 @@ static int consumer_stop( mlt_consumer parent );
 static int consumer_is_stopped( mlt_consumer parent );
 static void consumer_close( mlt_consumer parent );
 static void *consumer_thread( void * );
-static void consumer_refresh_cb( mlt_consumer sdl, mlt_consumer parent, char *name );
+static void consumer_refresh_cb(mlt_consumer sdl, mlt_consumer parent, mlt_event_data );
 static int jack_process( jack_nframes_t frames, void * data );
 
 /** Constructor
@@ -104,7 +104,7 @@ mlt_consumer consumer_jack_init( mlt_profile profile, mlt_service_type type, con
 
 			// Default scaler (for now we'll use nearest)
 			mlt_properties_set( properties, "rescale", "nearest" );
-			mlt_properties_set( properties, "deinterlace_method", "onefield" );
+			mlt_properties_set( properties, "consumer.deinterlacer", "onefield" );
 
 			// Default buffer for low latency
 			mlt_properties_set_int( properties, "buffer", 1 );
@@ -140,8 +140,9 @@ mlt_consumer consumer_jack_init( mlt_profile profile, mlt_service_type type, con
 	return NULL;
 }
 
-static void consumer_refresh_cb( mlt_consumer sdl, mlt_consumer parent, char *name )
+static void consumer_refresh_cb( mlt_consumer sdl, mlt_consumer parent, mlt_event_data event_data )
 {
+	const char *name = mlt_event_data_to_string(event_data);
 	if ( !strcmp( name, "refresh" ) )
 	{
 		consumer_jack self = parent->child;
@@ -362,8 +363,9 @@ static int consumer_play_video( consumer_jack self, mlt_frame frame )
 {
 	// Get the properties of this consumer
 	mlt_properties properties = MLT_CONSUMER_PROPERTIES( &self->parent );
-	if ( self->running && !mlt_consumer_is_stopped( &self->parent ) )
-		mlt_events_fire( properties, "consumer-frame-show", frame, NULL );
+	if ( self->running && !mlt_consumer_is_stopped( &self->parent ) ) {
+		mlt_events_fire( properties, "consumer-frame-show", mlt_event_data_from_frame(frame) );
+	}
 
 	return 0;
 }
