@@ -17,12 +17,47 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "common.h"
 #include <framework/mlt_filter.h>
 #include <framework/mlt_frame.h>
+#include <framework/mlt_slices.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 
+typedef struct {
+	uint8_t *image;
+	int width;
+	int height;
+	int noise;
+	double contrast;
+	double brightness;
+	mlt_position pos;
+	int min;
+	int max_luma;
+} slice_desc;
+
+static int slice_proc(int id, int index, int jobs, void* data)
+{
+	(void) id; // unused
+	slice_desc* d = (slice_desc*) data;
+	int slice_line_start, slice_height = mlt_slices_size_slice(jobs, index, d->height, &slice_line_start);
+	uint8_t *p = d->image + slice_line_start * d->width * 2;
+
+	oldfilm_rand_seed seed;
+	oldfilm_init_seed(&seed, d->pos * jobs + index);
+	for (int n = 0; n < slice_height * d->width; n++, p += 2) {
+		if (p[0] > 20) {
+			int pix = CLAMP(((double) p[0] - 127.0) * d->contrast + 127.0 + d->brightness, 0, 255);
+			if (d->noise > 0) {
+				pix -= oldfilm_fast_rand(&seed) % d->noise - d->noise;
+			}
+			p[0] = CLAMP(pix , d->min, d->max_luma);
+		}
+	}
+
+	return 0;
+}
 
 static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format *format, int *width, int *height, int writable )
 {
@@ -36,31 +71,20 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 
 	if ( error == 0 && *image )
 	{
-		int h = *height;
-		int w = *width;
-
-		double position = mlt_filter_get_progress( filter, frame );
-		srand(position*10000);
-
 		int noise = mlt_properties_anim_get_int( properties, "noise", pos, len );
-		double contrast = mlt_properties_anim_get_double( properties, "contrast", pos, len ) / 100.0;
-		double brightness = 127.0 * (mlt_properties_anim_get_double( properties, "brightness", pos, len ) -100.0 ) / 100.0;
-		
-		int x = 0,y = 0,pix = 0;
-		for ( x = 0; x < w; x++ )
-		{
-			for( y = 0; y < h; y++ )
-			{
-				uint8_t* pixel = (*image + (y) * w * 2 + (x) * 2 );
-				if (*pixel > 20)
-				{
-					pix = MIN ( MAX ( ( (double)*pixel -127.0  ) * contrast + 127.0 + brightness , 0 ) , 255 ) ;
-					if ( noise > 0 ) pix -= ( rand() % noise - noise );
-
-					*pixel = MIN ( MAX ( pix , 0 ) , 255 );
-				}
-			}
-		}
+		int full_range = mlt_properties_get_int(MLT_FRAME_PROPERTIES(frame), "full_range");
+		slice_desc desc = {
+			.image = *image,
+			.width = *width,
+			.height = *height,
+			.noise = noise,
+			.contrast = mlt_properties_anim_get_double( properties, "contrast", pos, len ) / 100.0,
+			.brightness = 127.0 * (mlt_properties_anim_get_double( properties, "brightness", pos, len ) -100.0 ) / 100.0,
+			.pos = pos,
+			.min = full_range? 0 : 16,
+			.max_luma = full_range? 255 : 235,
+		};
+		mlt_slices_run_normal(0, slice_proc, &desc);
 	}
 
 	return error;
