@@ -1,6 +1,6 @@
 /*
  * filter_text.c -- text overlay filter
- * Copyright (C) 2018-2019 Meltytech, LLC
+ * Copyright (C) 2018-2021 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,8 +21,10 @@
 #include <stdlib.h>
 #include <string.h>
 
-static void property_changed( mlt_service owner, mlt_filter filter, char *name )
+static void property_changed( mlt_service owner, mlt_filter filter, mlt_event_data event_data )
 {
+	const char *name = mlt_event_data_to_string(event_data);
+	if (!name) return;
 	if( !strcmp( "geometry", name ) ||
 		!strcmp( "family", name ) ||
 		!strcmp( "size", name ) ||
@@ -126,7 +128,7 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 
 		// Set the b_frame to be in the same position and have same consumer requirements
 		mlt_frame_set_position( b_frame, position );
-		mlt_properties_set_int( b_props, "consumer_deinterlace", mlt_properties_get_int( a_props, "consumer_deinterlace" ) );
+		mlt_properties_set_int( b_props, "consumer.progressive", mlt_properties_get_int( a_props, "consumer.progressive" ) );
 		mlt_properties_set_double( b_props, "consumer_scale", mlt_properties_get_double( a_props, "consumer_scale" ) );
 
 		// Apply all filters that are attached to this filter to the b frame
