@@ -1,6 +1,6 @@
 /*
  * transition_affine.c -- affine transformations
- * Copyright (C) 2003-2021 Meltytech, LLC
+ * Copyright (C) 2003-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -78,83 +78,6 @@ static double anim_get_angle(mlt_properties properties, const char* name, mlt_po
 	return result;
 }
 
-/** Calculate real geometry.
-*/
-
-static void geometry_calculate( mlt_transition transition, const char *store, struct mlt_geometry_item_s *output, double position )
-{
-	mlt_properties properties = MLT_TRANSITION_PROPERTIES( transition );
-	mlt_geometry geometry = mlt_properties_get_data( properties, store, NULL );
-	int mirror_off = mlt_properties_get_int( properties, "mirror_off" );
-	int repeat_off = mlt_properties_get_int( properties, "repeat_off" );
-	int length = mlt_geometry_get_length( geometry );
-
-	// Allow wrapping
-	if ( !repeat_off && position >= length && length != 0 )
-	{
-		int section = position / length;
-		position -= section * length;
-		if ( !mirror_off && section % 2 == 1 )
-			position = length - position;
-	}
-
-	// Fetch the key for the position
-	mlt_geometry_fetch( geometry, output, position );
-}
-
-
-static mlt_geometry transition_parse_keys( mlt_transition transition, const char *name, const char *store, int normalised_width, int normalised_height )
-{
-	// Get the properties of the transition
-	mlt_properties properties = MLT_TRANSITION_PROPERTIES( transition );
-
-	// Try to fetch it first
-	mlt_geometry geometry = mlt_properties_get_data( properties, store, NULL );
-
-	// Determine length and obtain cycle
-	mlt_position length = mlt_transition_get_length( transition );
-	double cycle = mlt_properties_get_double( properties, "cycle" );
-
-	// Allow a geometry repeat cycle
-	if ( cycle >= 1 )
-		length = cycle;
-	else if ( cycle > 0 )
-		length *= cycle;
-
-	if ( geometry == NULL )
-	{
-		// Get the new style geometry string
-		char *property = mlt_properties_get( properties, name );
-
-		// Create an empty geometries object
-		geometry = mlt_geometry_init( );
-
-		// Parse the geometry if we have one
-		mlt_geometry_parse( geometry, property, length, normalised_width, normalised_height );
-
-		// Store it
-		mlt_properties_set_data( properties, store, geometry, 0, ( mlt_destructor )mlt_geometry_close, NULL );
-	}
-	else
-	{
-		// Check for updates and refresh if necessary
-		mlt_geometry_refresh( geometry, mlt_properties_get( properties, name ), length, normalised_width, normalised_height );
-	}
-
-	return geometry;
-}
-
-static mlt_geometry composite_calculate( mlt_transition transition, struct mlt_geometry_item_s *result, int nw, int nh, double position )
-{
-	// Structures for geometry
-	mlt_geometry start = transition_parse_keys( transition, "geometry", "geometries", nw, nh );
-
-	// Do the calculation
-	geometry_calculate( transition, "geometries", result, position );
-
-	return start;
-}
-
 typedef struct
 {
 	double matrix[3][3];
@@ -404,13 +327,12 @@ static int sliced_proc( int id, int index, int jobs, void* cookie )
 {
 	(void) id; // unused
 	struct sliced_desc ctx = *((struct sliced_desc*) cookie);
-	int height_slice = (ctx.a_height + jobs / 2) / jobs;
-	int starty = height_slice * index;
+	int starty, height_slice = mlt_slices_size_slice(jobs, index, ctx.a_height, &starty);
 	double x, y;
 	double dx, dy;
 	int i, j;
 
-	ctx.a_image += (index * height_slice) * (ctx.a_width * 4);
+	ctx.a_image += starty * (ctx.a_width * 4);
 	for (i = 0, y = ctx.lower_y; i < ctx.a_height; i++, y++) {
 		if (i >= starty && i < (starty + height_slice)) {
 			for (j = 0, x = ctx.lower_x; j < ctx.a_width; j++, x++) {
@@ -447,7 +369,7 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 
 	// Image, format, width, height and image for the b frame
 	uint8_t *b_image = NULL;
-	mlt_image_format b_format = mlt_image_rgb24a;
+	mlt_image_format b_format = mlt_image_rgba;
 	int b_width = mlt_properties_get_int( b_props, "meta.media.width" );
 	int b_height = mlt_properties_get_int( b_props, "meta.media.height" );
 	double b_ar = mlt_frame_get_aspect_ratio( b_frame );
@@ -485,7 +407,7 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 	}
 	
 	// Fetch the a frame image
-	*format = mlt_image_rgb24a;
+	*format = mlt_image_rgba;
 	int error = mlt_frame_get_image( a_frame, image, format, width, height, 1 );
 	if (error || !image)
 		return error;
@@ -497,18 +419,7 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 
 	mlt_service_lock( MLT_TRANSITION_SERVICE( transition ) );
 
-	if (mlt_properties_get(properties, "geometry"))
-	{
-		// Structures for geometry
-		struct mlt_geometry_item_s geometry;
-		composite_calculate( transition, &geometry, normalised_width, normalised_height, ( double )position );
-		result.x = geometry.x;
-		result.y = geometry.y;
-		result.w = geometry.w;
-		result.h = geometry.h;
-		result.o = geometry.mix / 100.0f;
-	}
-	else if (mlt_properties_get(properties, "rect"))
+	if (mlt_properties_get(properties, "rect"))
 	{
 		// Determine length and obtain cycle
 		double cycle = mlt_properties_get_double( properties, "cycle" );
@@ -571,12 +482,11 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 				b_height = MAX(b_width * b_ar / b_dar, 1);
 			}
 			// Set the rescale interpolation to match the frame
-			mlt_properties_set( b_props, "rescale.interp", mlt_properties_get( a_props, "rescale.interp" ) );
+			mlt_properties_set( b_props, "consumer.rescale", mlt_properties_get( a_props, "consumer.rescale" ) );
 			// Disable padding (resize filter)
 			mlt_properties_set_int( b_props, "distort", 1 );
-	} else if (!mlt_properties_get_int(b_props, "interpolation_not_required")
-			   && (fill || distort || b_width > result.w || b_height > result.h
-				   || mlt_properties_get_int(properties, "b_scaled") || mlt_properties_get_int(b_props, "always_scale"))) {
+	} else if (mlt_properties_get_int(b_props, "always_scale") || (!mlt_properties_get_int(b_props, "interpolation_not_required")
+			  && (fill || distort || b_width > result.w || b_height > result.h || mlt_properties_get_int(properties, "b_scaled")))) {
 		// Request b frame image scaled to what is needed.
 		b_height = CLAMP(result.h, 1, MLT_AFFINE_MAX_DIMENSION);
 		b_width  = MAX(b_height * b_dar / b_ar, 1);
@@ -585,7 +495,7 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 			b_height = MAX(b_width * b_ar / b_dar, 1);
 		}
 		// Set the rescale interpolation to match the frame
-		mlt_properties_set( b_props, "rescale.interp", mlt_properties_get( a_props, "rescale.interp" ) );
+		mlt_properties_set( b_props, "consumer.rescale", mlt_properties_get( a_props, "consumer.rescale" ) );
 		// Disable padding (resize filter)
 		mlt_properties_set_int( b_props, "distort", 1 );
 	} else {
@@ -600,16 +510,16 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 		// Check if we are applied as a filter inside a transition
 		if (b_resource && !strcmp("<track>", b_resource)) {
 			// Set the rescale interpolation to match the frame
-			mlt_properties_set( b_props, "rescale.interp", mlt_properties_get( a_props, "rescale.interp" ) );
+			mlt_properties_set( b_props, "consumer.rescale", mlt_properties_get( a_props, "consumer.rescale" ) );
 		} else {
 			// Suppress padding and aspect normalization.
-			mlt_properties_set( b_props, "rescale.interp", "none" );
+			mlt_properties_set( b_props, "consumer.rescale", "none" );
 		}
 	}
 	mlt_log_debug(MLT_TRANSITION_SERVICE(transition), "requesting image B at resolution %dx%d\n", b_width, b_height);
 
 	// This is not a field-aware transform.
-	mlt_properties_set_int( b_props, "consumer_deinterlace", 1 );
+	mlt_properties_set_int( b_props, "consumer.progressive", 1 );
 
 	error = mlt_frame_get_image( b_frame, &b_image, &b_format, &b_width, &b_height, 0 );
 	if (error || !b_image) {
@@ -621,7 +531,7 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 	}
 
 	// Check that both images are of the correct format and process
-	if ( *format == mlt_image_rgb24a && b_format == mlt_image_rgb24a )
+	if ( *format == mlt_image_rgba && b_format == mlt_image_rgba )
 	{
 		double sw, sh;
 		// Get values from the transition
@@ -712,7 +622,7 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 		}
 
 
-		char *interps = mlt_properties_get( a_props, "rescale.interp" );
+		char *interps = mlt_properties_get( a_props, "consumer.rescale" );
 		// Copy in case string is changed.
 		if ( interps )
 			interps = strdup( interps );
