@@ -43,11 +43,9 @@
 #include <QWidget>
 #include <framework/mlt_log.h>
 
-#if QT_VERSION >= 0x040600
 #include <QGraphicsEffect>
 #include <QGraphicsBlurEffect>
 #include <QGraphicsDropShadowEffect>
-#endif
 
 #include <memory>
 
@@ -165,7 +163,7 @@ public:
 	}
 
 	void updateText(QString text) {
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 13, 0))
+#if QT_VERSION >= QT_VERSION_CHECK(5, 13, 0)
 		m_path.clear();
 #else
 		m_path = QPainterPath();
@@ -180,10 +178,18 @@ public:
 			linePos += m_lineSpacing;
 			if ( m_align == Qt::AlignHCenter )
 			{
+#if (QT_VERSION > QT_VERSION_CHECK(5, 11, 0))
+				double offset = (m_width - m_metrics.horizontalAdvance(line)) / 2;
+#else
 				double offset = (m_width - m_metrics.width(line)) / 2;
+#endif
 				linePath.translate(offset, 0);
 			} else if ( m_align == Qt::AlignRight ) {
+#if (QT_VERSION > QT_VERSION_CHECK(5, 11, 0))
+				double offset = (m_width - m_metrics.horizontalAdvance(line));
+#else
 				double offset = (m_width - m_metrics.width(line));
+#endif
 				linePath.translate(offset, 0);
 			}
 			m_path.addPath(linePath);
@@ -312,7 +318,7 @@ void loadFromXml( producer_ktitle self, QGraphicsScene *scene, const char *templ
 	// Check title locale
 	if ( title.hasAttribute( "LC_NUMERIC" ) ) {
 		QString locale = title.attribute( "LC_NUMERIC" );
-		QLocale::setDefault( locale );
+		QLocale::setDefault( QLocale( locale ) );
 	}
 
 	int originalWidth;
@@ -358,7 +364,7 @@ void loadFromXml( producer_ktitle self, QGraphicsScene *scene, const char *templ
 				else
 				{
 					// New: Font weight (QFont::)
-					font.setWeight( txtProperties.namedItem( "font-weight" ).nodeValue().toInt() );
+					font.setWeight( QFont::Weight( txtProperties.namedItem( "font-weight" ).nodeValue().toInt() ) );
 				}
 				font.setItalic( txtProperties.namedItem( "font-italic" ).nodeValue().toInt() );
 				font.setUnderline( txtProperties.namedItem( "font-underline" ).nodeValue().toInt() );
@@ -634,7 +640,6 @@ void loadFromXml( producer_ktitle self, QGraphicsScene *scene, const char *templ
 			int zValue = nodeAttributes.namedItem( "z-index" ).nodeValue().toInt();
 			gitem->setZValue( zValue );
 
-#if QT_VERSION >= 0x040600
 			// effects
 			QDomNode eff = items.item(i).namedItem("effect");
 			if (!eff.isNull()) {
@@ -651,7 +656,6 @@ void loadFromXml( producer_ktitle self, QGraphicsScene *scene, const char *templ
 					gitem->setGraphicsEffect(shadow);
 				}
 			}
-#endif
 		}
 	}
 
@@ -686,6 +690,18 @@ void loadFromXml( producer_ktitle self, QGraphicsScene *scene, const char *templ
 	return;
 }
 
+int initTitleProducer( mlt_producer producer )
+{
+	if ( !createQApplicationIfNeeded( MLT_PRODUCER_SERVICE(producer) ) )
+	{
+		return false;
+	}
+	if ( !QMetaType::type("QTextCursor") )
+	{
+		qRegisterMetaType<QTextCursor>( "QTextCursor" );
+	}
+	return true;
+}
 
 void drawKdenliveTitle( producer_ktitle self, mlt_frame frame, mlt_image_format format, int width, int height, double position, int force_refresh )
 {
@@ -726,12 +742,6 @@ void drawKdenliveTitle( producer_ktitle self, mlt_frame frame, mlt_image_format
 
 		if ( scene == NULL )
 		{
-			if ( !createQApplicationIfNeeded( MLT_PRODUCER_SERVICE(producer) ) ) {
-				pthread_mutex_unlock( &self->mutex );
-				return;
-			}
-			if ( !QMetaType::type("QTextCursor") )
-				qRegisterMetaType<QTextCursor>( "QTextCursor" );
 			scene = new QGraphicsScene();
 			scene->setItemIndexMethod( QGraphicsScene::NoIndex );
 			scene->setSceneRect(0, 0, mlt_properties_get_int( properties, "width" ), mlt_properties_get_int( properties, "height" ));
@@ -772,7 +782,7 @@ void drawKdenliveTitle( producer_ktitle self, mlt_frame frame, mlt_image_format
 
 		//must be extracted from kdenlive title
 		self->rgba_image = (uint8_t *) mlt_pool_alloc( image_size );
-#if QT_VERSION >= 0x050200
+#if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
 		// QImage::Format_RGBA8888 was added in Qt5.2
 		// Initialize the QImage with the MLT image because the data formats match.
 		QImage img( self->rgba_image, width, height, QImage::Format_RGBA8888 );
@@ -782,7 +792,11 @@ void drawKdenliveTitle( producer_ktitle self, mlt_frame frame, mlt_image_format
 		img.fill( 0 );
 		QPainter p1;
 		p1.begin( &img );
-		p1.setRenderHints( QPainter::Antialiasing | QPainter::TextAntialiasing | QPainter::HighQualityAntialiasing );
+		p1.setRenderHints( QPainter::Antialiasing | QPainter::TextAntialiasing
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+						   | QPainter::HighQualityAntialiasing
+#endif
+						   );
 		//| QPainter::SmoothPixmapTransform );
 		mlt_position anim_out = mlt_properties_get_position( producer_props, "_animation_out" );
 
@@ -811,7 +825,11 @@ void drawKdenliveTitle( producer_ktitle self, mlt_frame frame, mlt_image_format
 				img1.fill( 0 );
 				QPainter p2;
 				p2.begin(&img1);
-				p2.setRenderHints( QPainter::Antialiasing | QPainter::TextAntialiasing | QPainter::HighQualityAntialiasing );
+				p2.setRenderHints( QPainter::Antialiasing | QPainter::TextAntialiasing
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+									| QPainter::HighQualityAntialiasing
+#endif
+									);
 				scene->render(&p2,source,r2,  Qt::IgnoreAspectRatio );
 				p2.end();
 				int next_field_line = (  mlt_properties_get_int( producer_props, "top_field_first" ) ? 1 : 0 );
