@@ -98,10 +98,8 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 		// Clone the image and the alpha
 		int image_size = mlt_image_format_size( self->format, self->current_width, self->current_height, NULL );
 		uint8_t *image_copy = mlt_pool_alloc( image_size );
-		// We use height-1 because mlt_image_format_size() uses height + 1.
-		// XXX Remove -1 when mlt_image_format_size() is changed.
 		memcpy( image_copy, self->current_image,
-			mlt_image_format_size( self->format, self->current_width, self->current_height - 1, NULL ) );
+			mlt_image_format_size( self->format, self->current_width, self->current_height, NULL ) );
 		// Now update properties so we free the copy after
 		mlt_frame_set_image( frame, image_copy, image_size, mlt_pool_release );
 		// We're going to pass the copy on
@@ -150,6 +148,11 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 
 		// Set producer-specific frame properties
 		mlt_properties_set_int( properties, "progressive", mlt_properties_get_int( producer_props, "progressive" ) );
+
+		// Inform framework that this producer creates rgba frames by default
+		// TODO: read the producer's xml on opening so we know if we have RGB or RGBA data
+		mlt_properties_set_int( properties, "format", mlt_image_rgba );
+
 		double force_ratio = mlt_properties_get_double( producer_props, "force_aspect_ratio" );
 		if ( force_ratio > 0.0 )
 			mlt_properties_set_double( properties, "aspect_ratio", force_ratio );
@@ -193,6 +196,11 @@ mlt_producer producer_kdenlivetitle_init( mlt_profile profile, mlt_service_type
 		mlt_properties_set_int( properties, "progressive", 1 );
 		mlt_properties_set_int( properties, "aspect_ratio", 1 );
 		mlt_properties_set_int( properties, "seekable", 1 );
+		if ( !initTitleProducer( producer ) )
+		{
+			mlt_producer_close( producer );
+			return NULL;
+		}
 		read_xml(properties);
 		return producer;
 	}
