@@ -1,6 +1,6 @@
 /*
- * qimage_wrapper.cpp -- a QT/QImage based producer for MLT
- * Copyright (C) 2006-2021 Meltytech, LLC
+ * qimage_wrapper.cpp -- a Qt/QImage based producer for MLT
+ * Copyright (C) 2006-2022 Meltytech, LLC
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,8 +32,10 @@
 #include <QtEndian>
 #include <QTemporaryFile>
 #include <QImageReader>
+#include <QMovie>
 
 #ifdef USE_EXIF
+#include <QTransform>
 #include <libexif/exif-data.h>
 #endif
 
@@ -63,7 +65,7 @@ static void qimage_delete( void *data )
 
 }
 
-/// Returns false if this is animated.
+/// Returns frame count or 0 on error
 int init_qimage(mlt_producer producer, const char *filename)
 {
 	if (!createQApplicationIfNeeded(MLT_PRODUCER_SERVICE(producer))) {
@@ -74,7 +76,7 @@ int init_qimage(mlt_producer producer, const char *filename)
 	reader.setDecideFormatFromContent( true );
 	reader.setFileName( filename );
 	if ( reader.canRead() && reader.imageCount() > 1 ) {
-		return 0;
+		return reader.format() == "webp" ? reader.imageCount() : 0;
 	}
 #ifdef USE_KDE4
 	if ( !instance ) {
@@ -107,7 +109,7 @@ static QImage* reorient_with_exif( producer_qimage self, int image_idx, QImage *
 	{
 		  // Rotate image according to exif data
 		  QImage processed;
-		  QMatrix matrix;
+		  QTransform matrix;
 
 		  switch ( exif_orientation ) {
 		  case 2:
@@ -174,13 +176,44 @@ int refresh_qimage( producer_qimage self, mlt_frame frame, int enable_caching )
 	{
 		self->current_image = NULL;
 		QImageReader reader;
+		QImage *qimage;
+
 #if QT_VERSION >= QT_VERSION_CHECK(5, 5, 0)
 		// Use Qt's orientation detection
 		reader.setAutoTransform(!disable_exif);
 #endif
+		QString filename = QString::fromUtf8( mlt_properties_get_value( self->filenames, image_idx ) );
+		if (filename.isEmpty()) {
+			filename = QString::fromUtf8(mlt_properties_get(producer_props, "resource"));
+		}
+
+		// First try to detect the file type based on the content
+		// in case the file extension is incorrect.
 		reader.setDecideFormatFromContent( true );
-		reader.setFileName( QString::fromUtf8( mlt_properties_get_value( self->filenames, image_idx ) ) );
-		QImage *qimage = new  QImage( reader.read() );
+		reader.setFileName( filename );
+		if (reader.imageCount() > 1) {
+			QMovie movie(filename);
+			movie.setCacheMode(QMovie::CacheAll);
+			movie.jumpToFrame(image_idx);
+			qimage = new QImage(movie.currentImage());
+		} else {
+			qimage = new QImage(reader.read());
+		}
+		if ( qimage->isNull( ) )
+		{
+			mlt_log_info( MLT_PRODUCER_SERVICE( &self->parent ), "QImage retry: %d - %s\n",
+					reader.error(), reader.errorString().toLatin1().data() );
+			delete qimage;
+			// If detection fails, try a more comprehensive detection including file extension
+			reader.setDecideFormatFromContent( false );
+			reader.setFileName( filename );
+			qimage = new QImage( reader.read() );
+			if ( qimage->isNull( ) )
+			{
+				mlt_log_info( MLT_PRODUCER_SERVICE( &self->parent ), "QImage fail: %d - %s\n",
+						reader.error(), reader.errorString().toLatin1().data() );
+			}
+		}
 		self->qimage = qimage;
 
 		if ( !qimage->isNull( ) )
@@ -212,6 +245,7 @@ int refresh_qimage( producer_qimage self, mlt_frame frame, int enable_caching )
 			self->current_height = qimage->height( );
 
 			mlt_events_block( producer_props, NULL );
+			mlt_properties_set_int( producer_props, "format", qimage->hasAlphaChannel() ? mlt_image_rgba : mlt_image_rgb);
 			mlt_properties_set_int( producer_props, "meta.media.width", self->current_width );
 			mlt_properties_set_int( producer_props, "meta.media.height", self->current_height );
 			mlt_properties_set_int( producer_props, "_disable_exif", disable_exif );
@@ -279,7 +313,7 @@ void refresh_image( producer_qimage self, mlt_frame frame, mlt_image_format form
 
 		// Copy the image
 		int image_size;
-#if QT_VERSION >= 0x050200
+#if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
 		if ( has_alpha )
 		{
 			self->format = mlt_image_rgba;
@@ -357,7 +391,7 @@ void refresh_image( producer_qimage self, mlt_frame frame, mlt_image_format form
 				self->current_image = (uint8_t*) mlt_pool_alloc( image_size );
 				memcpy( self->current_image, buffer, image_size );
 			}
-			if ( ( buffer = (uint8_t*) mlt_properties_get_data( properties, "alpha", &self->alpha_size ) ) )
+			if ( ( buffer = (uint8_t*) mlt_frame_get_alpha_size(frame, &self->alpha_size) ) )
 			{
                 if ( !self->alpha_size )
                     self->alpha_size = self->current_width * self->current_height;
