@@ -1,8 +1,6 @@
 /*
  * qimage_wrapper.h -- a QT/QImage based producer for MLT
- *
- * NB: This module is designed to be functionally equivalent to the 
- * gtk2 image loading module so it can be used as replacement.
+ * Copyright (C) 2006-2021 Meltytech, LLC
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,6 +54,7 @@ extern void refresh_image( producer_qimage, mlt_frame, mlt_image_format, int wid
 extern void make_tempfile( producer_qimage, const char *xml );
 extern int init_qimage(mlt_producer producer, const char *filename);
 extern int load_sequence_sprintf( producer_qimage self, mlt_properties properties, const char *filename );
+extern int load_folder( producer_qimage self, const char *filename );
 
 
 #ifdef __cplusplus
