@@ -74,7 +74,7 @@ static void get_transform_config( VSTransformConfig* conf, mlt_filter filter, ml
 	}
 
 	// by default a bicubic interpolation is selected
-	const char *interps = mlt_properties_get( MLT_FRAME_PROPERTIES( frame ), "rescale.interp" );
+	const char *interps = mlt_properties_get( MLT_FRAME_PROPERTIES( frame ), "consumer.rescale" );
 	conf->interpolType = VS_BiCubic;
 	if ( strcmp( interps, "nearest" ) == 0 || strcmp( interps, "neighbor" ) == 0 )
 		conf->interpolType = VS_Zero;
@@ -347,7 +347,7 @@ static int get_image( mlt_frame frame, uint8_t **image, mlt_image_format *format
 	}
 
 	// VS only works on progressive frames
-	mlt_properties_set_int( MLT_FRAME_PROPERTIES( frame ), "consumer_deinterlace", 1 );
+	mlt_properties_set_int( MLT_FRAME_PROPERTIES( frame ), "consumer.progressive", 1 );
 
 	*format = validate_format( *format );
 
