@@ -1,6 +1,6 @@
 /*
  * producer_xml.c -- a libxml2 parser of mlt service networks
- * Copyright (C) 2003-2020 Meltytech, LLC
+ * Copyright (C) 2003-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -60,13 +60,16 @@ enum service_type
 	mlt_dummy_filter_type,
 	mlt_dummy_transition_type,
 	mlt_dummy_producer_type,
-	mlt_dummy_consumer_type
+	mlt_dummy_consumer_type,
+	mlt_chain_type,
+	mlt_link_type,
 };
 
 struct deserialise_context_s
 {
 	mlt_deque stack_types;
 	mlt_deque stack_service;
+	mlt_deque stack_properties;
 	mlt_properties producer_map;
 	mlt_properties destructors;
 	char *property;
@@ -180,6 +183,31 @@ static xmlNodePtr context_pop_node( deserialise_context context )
 	return mlt_deque_pop_back( context->stack_node );
 }
 
+// Get the current properties object to add each property to.
+// The object could be a service, or it could be a properties instance
+// nested under a service.
+static mlt_properties current_properties( deserialise_context context )
+{
+	mlt_properties properties = NULL;
+	if ( mlt_deque_count( context->stack_properties ) )
+	{
+		// Nested properties are being parsed
+		properties = mlt_deque_peek_back( context->stack_properties );
+	}
+	else if ( mlt_deque_count( context->stack_service ) )
+	{
+		// Properties belong to the service being parsed
+		mlt_service service = mlt_deque_peek_back( context->stack_service );
+		properties = MLT_SERVICE_PROPERTIES( service );
+	}
+
+	if ( properties )
+	{
+		mlt_properties_set_data( properties, "_profile", context->profile, 0, NULL, NULL );
+		mlt_properties_set_lcnumeric( properties, context->lc_numeric );
+	}
+	return properties;
+}
 
 // Set the destructor on a new service
 static void track_service( mlt_properties properties, void *service, mlt_destructor destructor )
@@ -442,8 +470,6 @@ static void on_start_tractor( deserialise_context context, const xmlChar *name,
 	for ( ; atts != NULL && *atts != NULL; atts += 2 )
 		mlt_properties_set_string( MLT_SERVICE_PROPERTIES( service ), (const char*) atts[0], atts[1] == NULL ? "" : (const char*) atts[1] );
 
-	mlt_properties_set_int( MLT_TRACTOR_PROPERTIES( tractor ), "global_feed", 1 );
-
 	if ( mlt_properties_get( properties, "id" ) != NULL )
 		mlt_properties_set_data( context->producer_map, mlt_properties_get( properties, "id" ), service, 0, NULL, NULL );
 	
@@ -571,6 +597,200 @@ static void on_end_playlist( deserialise_context context, const xmlChar *name )
 	}
 }
 
+static void on_start_chain( deserialise_context context, const xmlChar *name, const xmlChar **atts)
+{
+	mlt_chain chain = mlt_chain_init( context->profile );
+	mlt_service service = MLT_CHAIN_SERVICE( chain );
+	mlt_properties properties = MLT_SERVICE_PROPERTIES( service );
+
+	track_service( context->destructors, service, (mlt_destructor) mlt_chain_close );
+
+	for ( ; atts != NULL && *atts != NULL; atts += 2 )
+	{
+		mlt_properties_set_string( properties, (const char*) atts[0], atts[1] == NULL ? "" : (const char*) atts[1] );
+
+		// Out will be overwritten later as we append, so we need to save it
+		if ( xmlStrcmp( atts[ 0 ], _x("out") ) == 0 )
+			mlt_properties_set_string( properties, "_xml.out", ( const char* )atts[ 1 ] );
+	}
+
+	if ( mlt_properties_get( properties, "id" ) != NULL )
+		mlt_properties_set_data( context->producer_map, mlt_properties_get( properties, "id" ), service, 0, NULL, NULL );
+
+	context_push_service( context, service, mlt_chain_type );
+}
+
+static void on_end_chain( deserialise_context context, const xmlChar *name )
+{
+	// Get the chain from the stack
+	enum service_type type;
+	mlt_service service = context_pop_service( context, &type );
+
+	if ( service != NULL && type == mlt_chain_type )
+	{
+		mlt_chain chain = MLT_CHAIN( service );
+		mlt_properties properties = MLT_SERVICE_PROPERTIES( service );
+		mlt_position in = -1;
+		mlt_position out = -1;
+		mlt_producer source = NULL;
+
+		qualify_property( context, properties, "resource" );
+		char *resource = mlt_properties_get( properties, "resource" );
+
+		// Let Kino-SMIL src be a synonym for resource
+		if ( resource == NULL )
+		{
+			qualify_property( context, properties, "src" );
+			resource = mlt_properties_get( properties, "src" );
+		}
+
+		// Instantiate the producer
+		if ( mlt_properties_get( properties, "mlt_service" ) != NULL )
+		{
+			char *service_name = trim( mlt_properties_get( properties, "mlt_service" ) );
+			if ( resource )
+			{
+				// If a document was saved as +INVALID.txt (see below), then ignore the mlt_service and
+				// try to load it just from the resource. This is an attempt to recover the failed
+				// producer in case, for example, a file returns.
+				if (!strcmp("qtext", service_name)) {
+					const char *text = mlt_properties_get( properties, "text" );
+					if (text && !strcmp("INVALID", text)) {
+						service_name = NULL;
+					}
+				} else if (!strcmp("pango", service_name)) {
+					const char *markup = mlt_properties_get( properties, "markup" );
+					if (markup && !strcmp("INVALID", markup)) {
+						service_name = NULL;
+					}
+				}
+				if (service_name) {
+					char *temp = calloc( 1, strlen( service_name ) + strlen( resource ) + 2 );
+					strcat( temp, service_name );
+					strcat( temp, ":" );
+					strcat( temp, resource );
+					source = mlt_factory_producer( context->profile, NULL, temp );
+					free( temp );
+				}
+			}
+			else
+			{
+				source = mlt_factory_producer( context->profile, NULL, service_name );
+			}
+		}
+
+		// Just in case the plugin requested doesn't exist...
+		if ( !source && resource )
+			source = mlt_factory_producer( context->profile, NULL, resource );
+		if ( !source ) {
+			mlt_log_error( NULL, "[producer_xml] failed to load producer \"%s\"\n", resource );
+			source = mlt_factory_producer( context->profile, NULL, "+INVALID.txt" );
+			if (source) {
+				// Save the original mlt_service for the consumer to serialize it as original.
+				mlt_properties_set_string( properties, "_xml_mlt_service",
+					mlt_properties_get( properties, "mlt_service" ) );
+			}
+		}
+		if ( !source )
+			source = mlt_factory_producer( context->profile, NULL, "colour:red" );
+		// Propogate properties to the source
+		mlt_properties_inherit( MLT_PRODUCER_PROPERTIES( source ), properties );
+		// Add the source producer to the chain
+		mlt_chain_set_source( chain, source );
+
+		// See if the chain should be added to a playlist or multitrack
+		if ( mlt_properties_get( properties, "in" ) )
+			in = mlt_properties_get_position( properties, "in" );
+		if ( mlt_properties_get( properties, "out" ) )
+			out = mlt_properties_get_position( properties, "out" );
+		if ( add_producer( context, service, in, out ) == 0 )
+			context_push_service( context, service, type );
+	}
+	else
+	{
+		mlt_log_error( NULL, "[producer_xml] Invalid state of chain end %d\n", type );
+	}
+}
+
+static void on_start_link( deserialise_context context, const xmlChar *name, const xmlChar **atts)
+{
+	// Store properties until the service type is known
+	mlt_service service = calloc( 1, sizeof( struct mlt_service_s ) );
+	mlt_service_init( service, NULL );
+	mlt_properties properties = MLT_SERVICE_PROPERTIES( service );
+	context_push_service( context, service, mlt_link_type );
+
+	for ( ; atts != NULL && *atts != NULL; atts += 2 )
+		mlt_properties_set_string( properties, (const char*) atts[0], atts[1] == NULL ? "" : (const char*) atts[1] );
+}
+
+static void on_end_link( deserialise_context context, const xmlChar *name )
+{
+	enum service_type type;
+	mlt_service service = context_pop_service( context, &type );
+	mlt_properties properties = MLT_SERVICE_PROPERTIES( service );
+
+	enum service_type parent_type = mlt_invalid_type;
+	mlt_service parent = context_pop_service( context, &parent_type );
+
+	if ( service != NULL && type == mlt_link_type )
+	{
+		char *id = trim( mlt_properties_get( properties, "mlt_service" ) );
+		mlt_service link = MLT_SERVICE( mlt_factory_link( id, NULL ) );
+		mlt_properties link_props = MLT_SERVICE_PROPERTIES( link );
+
+		if ( !link )
+		{
+			mlt_log_error( NULL, "[producer_xml] failed to load link \"%s\"\n", id );
+			if ( parent )
+				context_push_service( context, parent, parent_type );
+			mlt_service_close( service );
+			free( service );
+			return;
+		}
+
+		track_service( context->destructors, link, (mlt_destructor) mlt_link_close );
+		mlt_properties_set_lcnumeric( MLT_SERVICE_PROPERTIES( link ), context->lc_numeric );
+
+		// Do not let XML overwrite these important properties set by mlt_factory.
+		mlt_properties_set_string( properties, "mlt_type", NULL );
+		mlt_properties_set_string( properties, "mlt_service", NULL );
+
+		// Propagate the properties
+		mlt_properties_inherit( link_props, properties );
+
+		// Attach the link to the chain
+		if ( parent != NULL )
+		{
+			if ( parent_type == mlt_chain_type )
+			{
+				mlt_chain_attach( MLT_CHAIN( parent ), MLT_LINK( link ) );
+			}
+			else
+			{
+				mlt_log_error( NULL, "[producer_xml] link can only be added to a chain...\n" );
+			}
+
+			// Put the parent back on the stack
+			context_push_service( context, parent, parent_type );
+		}
+		else
+		{
+			mlt_log_error( NULL, "[producer_xml] link closed with invalid parent...\n" );
+		}
+	}
+	else
+	{
+		mlt_log_error( NULL, "[producer_xml] Invalid top of stack on link close\n" );
+	}
+
+	if ( service )
+	{
+		mlt_service_close( service );
+		free(service);
+	}
+}
+
 static void on_start_producer( deserialise_context context, const xmlChar *name, const xmlChar **atts)
 {
 	// use a dummy service to hold properties to allow arbitrary nesting
@@ -1018,6 +1238,7 @@ static void on_end_filter( deserialise_context context, const xmlChar *name )
 		qualify_property( context, properties, "filename" );
 		qualify_property( context, properties, "av.file" );
 		qualify_property( context, properties, "av.filename" );
+		qualify_property( context, properties, "filter.resource" );
 		mlt_properties_inherit( filter_props, properties );
 
 		// Attach all filters from service onto filter
@@ -1228,7 +1449,7 @@ static void on_end_consumer( deserialise_context context, const xmlChar *name )
 					if ( !context->qglsl )
 					{
 						mlt_properties_pass_list( consumer_properties, properties,
-							"real_time, deinterlace_method, rescale, progressive, top_field_first, channels, channel_layout" );
+							"real_time, deinterlacer, deinterlace_method, rescale, progressive, top_field_first, channels, channel_layout" );
 
 						// We only really know how to optimize real_time for the avformat consumer.
 						const char *service_name = mlt_properties_get( properties, "mlt_service" );
@@ -1275,12 +1496,10 @@ static void on_end_consumer( deserialise_context context, const xmlChar *name )
 
 static void on_start_property( deserialise_context context, const xmlChar *name, const xmlChar **atts)
 {
-	enum service_type type;
-	mlt_service service = context_pop_service( context, &type );
-	mlt_properties properties = MLT_SERVICE_PROPERTIES( service );
+	mlt_properties properties = current_properties( context );
 	const char *value = NULL;
 
-	if ( service != NULL )
+	if ( properties != NULL )
 	{
 		// Set the properties
 		for ( ; atts != NULL && *atts != NULL; atts += 2 )
@@ -1296,26 +1515,22 @@ static void on_start_property( deserialise_context context, const xmlChar *name,
 
 		// Tell parser to collect any further nodes for serialisation
 		context->is_value = 1;
-
-		context_push_service( context, service, type );
 	}
 	else
 	{
-		mlt_log_error( NULL, "[producer_xml] Property without a service '%s'?\n", ( const char * )name );
+		mlt_log_error( NULL, "[producer_xml] Property without a parent '%s'?\n", ( const char * )name );
 	}
 }
 
 static void on_end_property( deserialise_context context, const xmlChar *name )
 {
-	enum service_type type;
-	mlt_service service = context_pop_service( context, &type );
-	mlt_properties properties = MLT_SERVICE_PROPERTIES( service );
+	mlt_properties properties = current_properties( context );
 
-	if ( service != NULL )
+	if ( properties != NULL )
 	{
 		// Tell parser to stop building a tree
 		context->is_value = 0;
-	
+
 		// See if there is a xml tree for the value
 		if ( context->property != NULL && context->value_doc != NULL )
 		{
@@ -1337,12 +1552,59 @@ static void on_end_property( deserialise_context context, const xmlChar *name )
 		// Close this property handling
 		free( context->property );
 		context->property = NULL;
-
-		context_push_service( context, service, type );
 	}
 	else
 	{
-		mlt_log_error( NULL, "[producer_xml] Property without a service '%s'??\n", (const char *)name );
+		mlt_log_error( NULL, "[producer_xml] Property without a parent '%s'??\n", (const char *)name );
+	}
+}
+
+static void on_start_properties( deserialise_context context, const xmlChar *name, const xmlChar **atts)
+{
+	mlt_properties parent_properties = current_properties( context );
+	if ( parent_properties != NULL )
+	{
+		mlt_properties properties = NULL;
+
+		// Get the name
+		for ( ; atts != NULL && *atts != NULL; atts += 2 )
+		{
+			if ( xmlStrcmp( atts[ 0 ], _x("name") ) == 0 )
+			{
+				properties = mlt_properties_new();
+				mlt_properties_set_properties( parent_properties, _s(atts[ 1 ]), properties );
+				mlt_properties_dec_ref( properties );
+			}
+			else
+			{
+				mlt_log_warning( NULL, "[producer_xml] Invalid attribute for properties '%s=%s'\n", ( const char * )atts[ 0 ], ( const char * )atts[ 1 ] );
+			}
+		}
+
+		if ( properties )
+		{
+			mlt_deque_push_back( context->stack_properties, properties );
+		}
+		else
+		{
+			mlt_log_error( NULL, "[producer_xml] Properties without a name '%s'?\n", ( const char * )name );
+		}
+	}
+	else
+	{
+		mlt_log_error( NULL, "[producer_xml] Properties without a parent '%s'?\n", ( const char * )name );
+	}
+}
+
+static void on_end_properties( deserialise_context context, const xmlChar *name )
+{
+	if ( mlt_deque_count( context->stack_properties ) )
+	{
+		mlt_deque_pop_back( context->stack_properties );
+	}
+	else
+	{
+		mlt_log_error( NULL, "[producer_xml] Properties end missing properties '%s'.\n", (const char *)name );
 	}
 }
 
@@ -1412,8 +1674,14 @@ static void on_start_element( void *ctx, const xmlChar *name, const xmlChar **at
 		on_start_filter( context, name, atts );
 	else if ( xmlStrcmp( name, _x("transition") ) == 0 )
 		on_start_transition( context, name, atts );
+	else if ( xmlStrcmp( name, _x("chain") ) == 0 )
+		on_start_chain( context, name, atts );
+	else if ( xmlStrcmp( name, _x("link") ) == 0 )
+		on_start_link( context, name, atts );
 	else if ( xmlStrcmp( name, _x("property") ) == 0 )
 		on_start_property( context, name, atts );
+	else if ( xmlStrcmp( name, _x("properties") ) == 0 )
+		on_start_properties( context, name, atts );
 	else if ( xmlStrcmp( name, _x("consumer") ) == 0 )
 		on_start_consumer( context, name, atts );
 	else if ( xmlStrcmp( name, _x("westley") ) == 0 || xmlStrcmp( name, _x("mlt") ) == 0 )
@@ -1447,12 +1715,18 @@ static void on_end_element( void *ctx, const xmlChar *name )
 		on_end_tractor( context, name );
 	else if ( xmlStrcmp( name, _x("property") ) == 0 )
 		on_end_property( context, name );
+	else if ( xmlStrcmp( name, _x("properties") ) == 0 )
+		on_end_properties( context, name );
 	else if ( xmlStrcmp( name, _x("producer") ) == 0 || xmlStrcmp( name, _x("video") ) == 0 )
 		on_end_producer( context, name );
 	else if ( xmlStrcmp( name, _x("filter") ) == 0 )
 		on_end_filter( context, name );
 	else if ( xmlStrcmp( name, _x("transition") ) == 0 )
 		on_end_transition( context, name );
+	else if ( xmlStrcmp( name, _x("chain") ) == 0 )
+		on_end_chain( context, name );
+	else if ( xmlStrcmp( name, _x("link") ) == 0 )
+		on_end_link( context, name );
 	else if ( xmlStrcmp( name, _x("consumer") ) == 0 )
 		on_end_consumer( context, name );
 
@@ -1464,12 +1738,7 @@ static void on_characters( void *ctx, const xmlChar *ch, int len )
 	struct _xmlParserCtxt *xmlcontext = ( struct _xmlParserCtxt* )ctx;
 	deserialise_context context = ( deserialise_context )( xmlcontext->_private );
 	char *value = calloc( 1, len + 1 );
-	enum service_type type;
-	mlt_service service = context_pop_service( context, &type );
-	mlt_properties properties = MLT_SERVICE_PROPERTIES( service );
-
-	if ( service != NULL )
-		context_push_service( context, service, type );
+	mlt_properties properties = current_properties( context );
 
 	value[ len ] = 0;
 	strncpy( value, (const char*) ch, len );
@@ -1753,6 +2022,7 @@ static deserialise_context context_new( mlt_profile profile )
 		context->seekable = 1;
 		context->stack_service = mlt_deque_init();
 		context->stack_types = mlt_deque_init();
+		context->stack_properties = mlt_deque_init();
 		context->stack_node = mlt_deque_init();
 		context->stack_branch = mlt_deque_init();
 		mlt_deque_push_back_int( context->stack_branch, 0 );
@@ -1767,6 +2037,7 @@ static void context_close( deserialise_context context )
 	mlt_properties_close( context->params );
 	mlt_deque_close( context->stack_service );
 	mlt_deque_close( context->stack_types );
+	mlt_deque_close( context->stack_properties );
 	mlt_deque_close( context->stack_node );
 	mlt_deque_close( context->stack_branch );
 	xmlFreeDoc( context->entity_doc );
@@ -1911,9 +2182,11 @@ mlt_producer producer_xml_init( mlt_profile profile, mlt_service_type servtype,
 	// Reset the stack.
 	mlt_deque_close( context->stack_service );
 	mlt_deque_close( context->stack_types );
+	mlt_deque_close( context->stack_properties );
 	mlt_deque_close( context->stack_node );
 	context->stack_service = mlt_deque_init();
 	context->stack_types = mlt_deque_init();
+	context->stack_properties = mlt_deque_init();
 	context->stack_node = mlt_deque_init();
 
 	// Create the qglsl consumer now, if requested, so that glsl.manager
@@ -1956,9 +2229,9 @@ mlt_producer producer_xml_init( mlt_profile profile, mlt_service_type servtype,
 	if ( well_formed && service != NULL )
 	{
 		// Verify it is a producer service (mlt_type="mlt_producer")
-		// (producer, playlist, multitrack)
+		// (producer, chain, playlist, multitrack)
 		char *type = mlt_properties_get( MLT_SERVICE_PROPERTIES( service ), "mlt_type" );
-		if ( type == NULL || ( strcmp( type, "mlt_producer" ) != 0 && strcmp( type, "producer" ) != 0 ) )
+		if ( type == NULL || ( strcmp( type, "mlt_producer" ) != 0 && strcmp( type, "producer" ) != 0 && strcmp( type, "chain" ) != 0 ) )
 			service = NULL;
 	}
 
@@ -2002,9 +2275,9 @@ mlt_producer producer_xml_init( mlt_profile profile, mlt_service_type servtype,
 		{
 			// Now assign additional properties
 			if ( is_filename && (
-				mlt_service_identify( service ) == tractor_type ||
-				mlt_service_identify( service ) == playlist_type ||
-				mlt_service_identify( service ) == multitrack_type ) )
+				mlt_service_identify( service ) == mlt_service_tractor_type ||
+				mlt_service_identify( service ) == mlt_service_playlist_type ||
+				mlt_service_identify( service ) == mlt_service_multitrack_type ) )
 			{
 				mlt_properties_set_int( properties, "_original_type",
 					mlt_service_identify( service ) );
